/*
*#######################################################################
* Copyright (C) 2008-2013 TES Electronic Solutions GmbH,
* All Rights Reserved.
* This source code and any compilation or derivative thereof is the
* proprietary information of TES Electronic Solutions GmbH
* and is confidential in nature.
* Under no circumstances is this software to be exposed to or placed
* under an Open Source License of any type without the expressed
* written permission of TES Electronic Solutions GmbH
*
*#######################################################################
*/

#ifndef __TES_GLYPHLIB_H
#define __TES_GLYPHLIB_H

#if defined(__GNUC__)
#define GUI_ATTRIBUTE_PACKED __attribute__ ((__packed__))
#elif defined(_MSC_VER)
#define GUI_ATTRIBUTE_PACKED
#elif defined(__RX)
#elif defined(__CC_ARM)
#define GUI_ATTRIBUTE_PACKED __packed
#else
#define GUI_ATTRIBUTE_PACKED
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/* Font-Library version for compile time asserts */
#define FL_LIBRARY_VERSION  5

    typedef enum
    {
        FL_ERR_OKAY,
        FL_ERR_FILE_NOT_FOUND,
        FL_ERR_OUT_OF_MEMORY,
        FL_ERR_VERSION_MISMATCH,
        FL_ERR_FILE_CORRUPT,
        FL_ERR_INVALID_HANDLE,
        FL_ERR_INVALID_PARAMETER,
    } FL_ERRORCODES;

    typedef enum
    {
        FL_FALSE = 0,
        FL_TRUE = 1
    } FL_Bool;

    /*-----------------------------------
    Opaque Handle Type-Definitions:
    -----------------------------------*/
    typedef void * FL_LibHandle; /* Opaque Library Handle  */
    typedef void * FL_FontHandle; /* Opaque Font Handle */
    typedef int FL_GlyphId; /* Glyph-Id */

/* Id of non-existent Glyph-ID's */
#define FL_NO_GLYPH -1

    /*-----------------------------------
    Client-Side Structures:
    -----------------------------------*/
    typedef struct
    {
        int BoxWidth; /* Glyph box width */
        int BoxHeight; /* Glyph box height */
        int PlaceX; /* placing w.r.t the current doDraw position */
        int PlaceY; /* placing w.r.t the current doDraw position */
        int AdvanceX; /* step in x for the next character */
    } FL_GlyphInfo;

    /*----------------------------------
    Font Metrics Structure. Directly
    taken from freetype
    -----------------------------------*/
    GUI_ATTRIBUTE_PACKED struct FL_FontMetrics_s
    {
        int ascender; /* Distance from baseline to highest coordinate (26.6 format) */
        int descender; /* Distance from baseline to lowest coordinate (26.6 format) */
        int height; /* Font-Height (integer) */
        int maxadvance; /* max. advance in X (26.6 format) */
        int x_ppem; /* em-width, integer */
        int y_ppem; /* em-height, integer */
        int em_units; /* em_units (since v5) */
    };
    typedef struct FL_FontMetrics_s FL_FontMetrics;

    GUI_ATTRIBUTE_PACKED struct FontHeader_s
    {
        int Version; /* Version Tag */
        int GlyphCount; /* Glyph-Count */
        GUI_ATTRIBUTE_PACKED FL_FontMetrics Metrics; /* Truetype Metrics. */
        int GlyphInfoOffset; /* File-Offset to GlyphInfo bitstream */
        int BoxWidthBits; /* bit-count for boxwidth */
        int BoxHeightBits; /* bit-count for boxheight */
        int PlaceXBits; /* bit-count for placex */
        int PlaceYBits; /* bit-count for placey */
        int AdvanceBits; /* bit-count for advancex */
        int ImageOffsetBits; /* bit-count for image offset */
        int FaceNameOffset;
        /* if Version >= 3, the following field(s) become valid */
        int KerningOffset;
        /* if Version >= 4, the followings fields become valid */
        int MaxBoxWidth;
        int MaxBoxHeight;
        /* if Version >= 6, the following field becomes valid */
        int CompressedGlyphs;
    };
    typedef struct FontHeader_s FontHeader;

    /*---------------------------------
    Kerning structs
    ----------------------------------*/
    GUI_ATTRIBUTE_PACKED struct FL_KernInfo_s
    {
        signed short kern_x;
        signed short kern_y;
    };
    typedef struct FL_KernInfo_s FL_KernInfo;

    GUI_ATTRIBUTE_PACKED struct FL_KernEntry_s
    {
        unsigned short left_unicode;
        unsigned short right_unicode;
        unsigned short kerninfo_idx;
        unsigned short __padding;
    };
    typedef struct FL_KernEntry_s FL_KernEntry;

    GUI_ATTRIBUTE_PACKED struct Font_s
    {
        const FontHeader * m_Header; /* Header structure */
        unsigned int sizeof_fontblob; /* size of the row font-data buffer */
        int m_KerningBits; /* number of bits per kerning-entry: */
        unsigned int sizeof_fontheader;

        GUI_ATTRIBUTE_PACKED struct
        {
            unsigned int num_infos;
            const FL_KernInfo * infos;
            unsigned int num_entries;
            const FL_KernEntry *entries;
        } kerning;

        const unsigned short * m_Directory; /* Pointer to Glyph-Directory */
        const unsigned char * m_FontBlob; /* Pointer to raw font-data */
        unsigned short max_box_width; /* new in v4 */
        unsigned short max_box_height;
        unsigned char compression;
    };
    typedef struct Font_s Font;

    GUI_ATTRIBUTE_PACKED struct LibContext_s
    {
        FL_ERRORCODES m_ErrorCode; /* Current error-state: */
    };
    typedef struct LibContext_s LibContext;

    /*------------------------------------
    Library API Entrypoints:
    -------------------------------------*/

    /**
    @brief Initialize a glyph-library
    */
    FL_Bool FL_CreateFontLibrary(void);

    /**
    @brief Get the version-number_
    */
    int FL_GetVersionNumber(void);

    /**
    @brief Get an error-code.
    */
    FL_ERRORCODES FL_GetError(FL_LibHandle a_Handle);

    /*-----------------------------------
    Font-Handling:
    ------------------------------------*/

    /**
    @brief return library handle

    @return internal library-handle.
    */
    FL_LibHandle FL_GetHandle();

    /**
    @brief Load a font from file:
    NOTE: a_Font will be initialized

    @param a_Lib        Library handle
    @param a_data        Data
    @param numBytes    Number of bytes

    @return Font handle
    */
    FL_FontHandle FL_LoadMemoryFont(
        FL_LibHandle a_Lib,
        const char *a_data,
        unsigned int numBytes
    );

    /**
    @brief Unload a font from memory

    @param fh           handle of font
    */
    void FL_UnloadMemoryFont(FL_FontHandle a_Font);

    /**
    @brief Get Font-Metrics:

    @param a_Lib        Library handle
    @param a_Font        Font handle
    @param a_Metrics    Metrics handle
    */
    void FL_GetFontMetrics(
        FL_LibHandle a_Lib,
        FL_FontHandle a_Font,
        FL_FontMetrics * a_Metrics
    );

    /**
    @brief Get maximum glyph box extents (v4)

    @param a_Lib                Library handle
    @param a_Font                Font handle
    @param a_MaxBoxWidthRet        Max Box Width
    @param a_MaxBoxHeightRet    Max Box Height
    */
    void FL_GetMaxExtents(
        FL_LibHandle a_Lib,
        FL_FontHandle a_Font,
        unsigned short *a_MaxBoxWidthRet,
        unsigned short *a_MaxBoxHeightRet
    );

    /**
    @brief Get the face name.

    @param a_Lib    Library handle
    @param a_Font    Font handle

    @return Face name
    */
    const char * FL_GetFontFaceName(
        FL_LibHandle a_Lib,
        FL_FontHandle a_Font
    );

    /**
    @brief Get compression of font
    
    @param a_Lib    Library handle
    @param a_Font    Font handle
    @param compression 
    */
    void FL_GetFontCompression(
        FL_LibHandle a_Lib,
        FL_FontHandle a_Font,
        unsigned char* compression);

    /*-----------------------------------
    Glyph Handling:
    ------------------------------------*/

    /**
    @brief Get glyph index for an unicode-char (or -1 if the char does not exist)

    @param a_Lib            Library handle
    @param a_Font            Font handle
    @param a_UnicodeChar    Unicode character

    @return Glyph ID
    */
    FL_GlyphId FL_GetGlyphIndex(
        FL_LibHandle a_Lib,
        FL_FontHandle a_Font,
        unsigned short a_UnicodeChar
    );

    /**
    @brief Get the glyph-count within a font.

    @param a_Lib    Library handle
    @param a_Font    Font handle

    @return Number of glyphs within a font
    */
    int FL_GetGlyphCount(
        FL_LibHandle a_Lib,
        FL_FontHandle a_Font
    );

    /**
    @brief Get placement and kerning-info of a glyph:

    @param a_Lib        Library handle
    @param a_Font        Font handle
    @param a_GlyphIndex    Glyph index
    @param a_Info        Glyph info

    @return True on success, false otherwise
    */
    FL_Bool FL_GetGlyphInfo(
        FL_LibHandle a_Lib,
        FL_FontHandle a_Font,
        FL_GlyphId a_GlyphIndex,
        FL_GlyphInfo * a_Info
    );

    /**
    @brief Get image of a glyph (4 bits per pixel):

    @param a_Lib        Library handle
    @param a_Font        Font handle
    @param a_GlyphIndex    Glyph index

    @return Glyph Bitmap Address
    */
    const unsigned char *FL_GetGlyphBitmapAddr(
        FL_LibHandle a_Lib,
        FL_FontHandle a_Font,
        FL_GlyphId a_GlyphIndex
    );

    /**
    @brief Query kerning info for a given glyph pair
    retX/retY will received glyph offset in 1:9:6 fixpoint format

    @param lib            Library handle
    @param font        Font handle
    @param unicode        Unicode
    @param unicodeNext    Next unicode
    @param retX        X offset
    @param retY        Y offset

    @return True on success, false otherwise
    */
    FL_Bool FL_GetKernInfo(FL_LibHandle lib,
        FL_FontHandle font,
        unsigned short unicode,
        unsigned short unicodeNext,
        signed short *retX,
        signed short *retY
    );

#ifdef __cplusplus
}
#endif

#endif
