#if !defined(ONSCREENKEYBOARD_H)
#define ONSCREENKEYBOARD_H

#include "GUIObject.h"

#include "GUIOnscreenKeyboardLayout.h"

#include "GUIColorPropertyObserver.h"
#include "GUIImageResource.h"
#include "GUIFontResource.h"
#include "GUIObjectHandleResource.h"
#include "GUIEdit.h"
#include "GUINinePatch.h"
#include "GUIGeneralResourceManager.h"

/**
@brief This class represents an onscreenkeyboard

@ingroup GUILIANI_CONTROLS
*/
class CGUIOnscreenKeyboard : public CGUIObject, public CGUIColorPropertyObserver
{
public:
    /// selected button
    struct SelectedButton_t
    {
        /** default constructor
        */
        SelectedButton_t() :
            bSelected(false),
            uiLineIndex(0),
            uiKeyIndex(0),
            bSubCharsSelected(false),
            uiSubCharLineIndex(0),
            uiSubCharsKeyIndex(0)
        {}

        /** set selected button
        @param LineIndex index of line
        @param KeyIndex index of key
        */
        void Set(const eC_UInt& LineIndex, const eC_UInt& KeyIndex)
        {
            bSelected = true;
            uiLineIndex = LineIndex;
            uiKeyIndex = KeyIndex;
        }

        /** reset selected button
        */
        void Reset()
        {
            bSelected = false;
            uiLineIndex = 0;
            uiKeyIndex = 0;
            bSubCharsSelected = false;
            uiSubCharLineIndex = 0;
            uiSubCharsKeyIndex = 0;
        }

        /** set subchar-index for selected button
        @param LineIndex index of line
        @param KeyIndex index of key
        */
        void SetSubChars(const eC_UInt& LineIndex, const eC_UInt& KeyIndex)
        {
            bSubCharsSelected = true;
            uiSubCharLineIndex = LineIndex;
            uiSubCharsKeyIndex = KeyIndex;
        }

        /** check for specific index
        @param LineIndex index of line
        @param KeyIndex index of key
        @return true if equal
        */
        eC_Bool Equals(const eC_UInt& LineIndex, const eC_UInt& KeyIndex)
        {
            if (bSelected)
            {
                if ((LineIndex == uiLineIndex) && (KeyIndex == uiKeyIndex))
                    return true;
            }
            return false;
        }

        eC_Bool bSelected; ///< button selected
        eC_UInt uiLineIndex; ///< index of line
        eC_UInt uiKeyIndex; ///< index of key
        eC_Bool bSubCharsSelected; ///< subchar selected
        eC_UInt uiSubCharLineIndex; ///< index of line in subchar
        eC_UInt uiSubCharsKeyIndex; ///< index of key in subchar
    };

public:
    /** constructor.
    @param pkParent Pointer to the designated parent object
    @param vX X-position relative to its parent object
    @param vY Y-position relative to its parent object
    @param vWidth Width of the object
    @param vHeight Height of the object
    @param eID of this object
    */
    CGUIOnscreenKeyboard(
        CGUICompositeObject* const pkParent,
        const eC_Value& vX, const eC_Value& vY,
        const eC_Value& vWidth, const eC_Value& vHeight,
        const ObjectHandle_t &eID = NO_HANDLE);

    /** default constructor
    */
    CGUIOnscreenKeyboard();

    /** destructor
    */
    ~CGUIOnscreenKeyboard();

    /**
    Load a keyboard-layout using general resources
    @param reResource the ID of the resource to use
    */
    void LoadLayout(const GeneralResource_t& reResource);

    /**
    Load a keyboard-layout from a file
    @param pkFile file to load from
    @param kFileExtension extension of file to choose correct loader
    @return true on success, otherwise false
    */
    eC_Bool LoadLayout(eC_File* pkFile, const eC_String& kFileExtension);

    /**
    Load a keyboard-layout from a file
    @param rkLayoutFile
    @return true on success, otherwise false
    */
    eC_Bool LoadLayout(const eC_String& rkLayoutFile);

    /** get the currently set layout
    @return layout
    */
    CGUIOnscreenKeyboardLayout* GetLayout() const;

    /** set the keyboard-layout
    @param pkLayout
    */
    void SetLayout(CGUIOnscreenKeyboardLayout* pkLayout);

    // attributes
    /** Set font
    @param eFont
    */
    void SetFont(const FontResource_t& eFont);

    /** Get font
    @return font
    */
    FontResource_t GetFont() const;

    /** Set font color as property
    @param eProperty
    */
    void SetFontColor(const GlobalProperty_t& eProperty);

    /** Set font-color as color
    @param uiColor
    */
    void SetFontColor(const eC_UInt& uiColor);

    /** Get font-color
    @return font-color
    */
    eC_UInt GetFontColor() const;

    /** Set selected font-color as property
    @param eProperty
    */
    void SetFontColorSelected(const GlobalProperty_t& eProperty);

    /** Set selected font-color as color
    @param uiColor
    */
    void SetFontColorSelected(const eC_UInt& uiColor);

    /** Get selected font-color
    @return color
    */
    eC_UInt GetFontColorSelected() const;

    /** Set background-image
    @param eImage
    */
    void SetBackgroundImage(const ImageResource_t& eImage);

    /** Get background-image
    @return background-image
    */
    ImageResource_t GetBackgroundImage() const;

    /** Set background-color as property
    @param eProperty
    */
    void SetBackgroundColor(const GlobalProperty_t& eProperty);

    /** Set background-color as color
    @param uiColor
    */
    void SetBackgroundColor(const eC_UInt& uiColor);

    /** Get background-color
    @return background-color
    */
    eC_UInt GetBackgroundColor() const;

    /** Set selected background image
    @param eImage
    */
    void SetBackgroundImageSelected(const ImageResource_t& eImage);

    /** Get selected background-image
    @return image
    */
    ImageResource_t GetBackgroundImageSelected() const;

    /** Set selected background-color as property
    @param eProperty
    */
    void SetBackgroundColorSelected(const GlobalProperty_t& eProperty);

    /** Set selected background-color as color
    @param uiColor
    */
    void SetBackgroundColorSelected(const eC_UInt& uiColor);

    /** Get selected background-color
    @return color
    */
    eC_UInt GetBackgroundColorSelected() const;

    /** Set overlay-color as property
    @param eProperty
    */
    void SetOverlayColor(const GlobalProperty_t& eProperty);

    /** Set overlay-color as color
    @param uiColor
    */
    void SetOverlayColor(const eC_UInt& uiColor);

    /** Get overlay-color
    @return color
    */
    eC_UInt GetOverlayColor() const;

    /** Set overlay image
    @param eImage
    */
    void SetOverlayImage(const ImageResource_t& eImage);

    /** Get overlay-image
    @return image
    */
    ImageResource_t GetOverlayImage() const;

    /** Get ninepatch for images
    @return nine-patch
    */
    CGUINinePatch& GetNinePatch();

    /** Set objectID of target editfield
    @param eObjectID of editfield
    */
    void SetTargetObject(const ObjectHandle_t& eObjectID);

    /** Get ObjectID of editfield
    @return objectID
    */
    ObjectHandle_t GetTargetObject() const;

    /** set to use overlay
    @param bUseOverlay
    */
    void SetUseOverlay(const eC_Bool& bUseOverlay);

    /** get use overlay
    @return true if active
    */
    eC_Bool GetUseOverlay() const;

    /** set to allow subchars
    @param bAllowSubChars
    */
    void SetAllowSubChars(const eC_Bool& bAllowSubChars);

    /** get allow subchars
    @return true if allowed
    */
    eC_Bool GetAllowSubChars() const;

    /** Set index of currently active page
    @param ubPageIndex
    */
    void SetPageIndex(const eC_UByte& ubPageIndex);

    /** Get index of currently activee page
    @return index
    */
    eC_UInt GetPageIndex() const;

    virtual void SetWidth(const eC_Value& vWidth);
    virtual void SetHeight(const eC_Value& vHeight);

    virtual void DoAnimate(const eC_Value& vTimes = eC_FromInt(1));

    virtual eC_Bool DoDraw();

    virtual eC_Bool DoButtonDown(
        const eC_Value& vAbsX = eC_FromInt(-1),
        const eC_Value& vAbsY = eC_FromInt(-1));

    virtual eC_Bool DoButtonUp(
        const eC_Value& vAbsX = eC_FromInt(-1),
        const eC_Value& vAbsY = eC_FromInt(-1));

    virtual eC_Bool DoDrag(
        const eC_Value& vDeltaX = eC_FromInt(-1),
        const eC_Value& vDeltaY = eC_FromInt(-1),
        const eC_Value& vAbsX = eC_FromInt(-1),
        const eC_Value& vAbsY = eC_FromInt(-1));

#if defined(GUILIANI_STREAM_GUI)
    /** Reads all attributes from streaming file.
        This method is called by CGUIFactoryManager after one of the registered
        factories has created an instance of this class.
    */
    virtual void ReadFromStream();
#endif

#if defined(GUILIANI_WRITE_GUI)
    /** Writes all attributes to the streaming file. A CGUIStreamWriter
        has to be initialized first.
        @param bWriteClassID This flag is used to select if writing of control
               class ID, leading and trailing tags is performed.
    */
    virtual void WriteToStream(const eC_Bool bWriteClassID = false);
#endif

private:
    void Init();
    void DeInit();

    void DrawKey(
        eC_String* pkString,
        const eC_Value& vPosX,
        const eC_Value& vPosY,
        const eC_Value& vWidth,
        const eC_Value& vHeight,
        const eC_UInt& uiColor);

    void GetObject();

    eC_Bool GetKeyIndex(
        const eC_Value& vAbsX, const eC_Value& vAbsY,
        eC_Bool& bSubChars,
        eC_UInt& ruiLineIndex, eC_UInt& ruiKeyIndex) const;

    CGUIOnscreenKeyboardLayout::KeyboardKey_t* GetSelectedKey() const;

    void InsertKey();

    void StartTimer(const eC_UInt& uiInterval);
    void StopTimer();

    void ExpandKeyboard(const eC_Value& vNeededHeight);
    void ResetKeyboard();

    void DrawSelectedBackground(const CGUIRect& kRect);

private:
    static const eC_UInt INDEX_FONT_COLOR;
    static const eC_UInt INDEX_FONT_COLOR_SELECTED;
    static const eC_UInt INDEX_BACKGROUND_COLOR;
    static const eC_UInt INDEX_BACKGROUND_COLOR_SELECTED;
    static const eC_UInt INDEX_BACKGROUND_COLOR_OVERLAY;

    GeneralResource_t m_eLayout;

    FontResource_t m_eFont;
    ImageResource_t m_eImage;
    ImageResource_t m_eImageSelected;
    ImageResource_t m_eImageOverlay;

    CGUINinePatch m_kNinePatch;

    ObjectHandle_t m_eTargetObject;
    CGUIEdit* m_pkEdit;

    eC_UByte m_ubCurrentPageIndex;
    CGUIOnscreenKeyboardLayout* m_pkLayout;

    eC_Bool m_bShowSubChars;
    CGUIRect m_kSubCharsRect;
    eC_Bool m_bButtonDown;
    SelectedButton_t m_kLastSelectedButton;

    eC_Bool m_bUseOverlay;
    eC_Bool m_bAllowSubChars;
    eC_Bool m_bExpanded;
    eC_Value m_vOriginalYPos;
    eC_Value m_vOriginalHeight;
    eC_Bool m_bTimerTriggered;
};

/** This class is used to interpret the layout-file on loading
*/
class OnScreenKeyboardLayoutInterpreter : public CGUIGeneralResourceInterpreter
{
public:
    /** Constructor
    @param pkObject target-keyboard
    @param kFileExtension extension of the file (xml or bin)
    */
    OnScreenKeyboardLayoutInterpreter(CGUIOnscreenKeyboard* pkObject, const eC_String& kFileExtension) :
        CGUIGeneralResourceInterpreter(),
        m_pkObject(pkObject),
        m_kFileExtension(kFileExtension)
    {}

    /** interpret the resource-file containing the keyboard-layout
    @param pkFile input-file
    */
    virtual void Interpret(eC_File* pkFile)
    {
        if (NULL != m_pkObject)
            m_pkObject->LoadLayout(pkFile, m_kFileExtension);
    }

private:
    CGUIOnscreenKeyboard* m_pkObject; ///< attached keyboard
    eC_String m_kFileExtension; ///< extension of file
};

#endif
