#ifndef __ONSCREEN_KEYBOARD_LAYOUT_H__
#define __ONSCREEN_KEYBOARD_LAYOUT_H__

#include "eC_TArray.h"
#include "eC_String.h"

#include "GUIRect.h"

/**
@brief Layout for an OnScreenKeyboard
*/
class CGUIOnscreenKeyboardLayout
{
public:
    /// type of key
    enum KeyboardKeyType_t
    {
        KT_UNKNOWN = 0,
        KT_PRINTABLE_KEY,
        KT_SPECIAL_KEY,
        KT_SEPARATOR,
        KT_PAGE_UPPERCASE,
        KT_PAGE_LOWERCASE,
        KT_PAGE_NUMBERS,
        KT_PAGE_SYMBOLS
    };

    /// forward declaration for subchars
    struct KeyboardSubChars_t;

    /// struct for a key
    struct KeyboardKey_t
    {
        /** default constructor
        */
        KeyboardKey_t() :
            eType(KT_PRINTABLE_KEY),
            kString(""),
            uiKeyCode(0),
            ubWidth(1),
            pkSubChars(NULL)
        {}

        /** constructor
        @param Type
        @param Text
        @param Code
        @param Width
        @param SubChars
        */
        KeyboardKey_t(
            const KeyboardKeyType_t& Type,
            const eC_String& Text,
            const eC_UInt& Code,
            const eC_UByte& Width,
            KeyboardSubChars_t* SubChars = NULL) :
            eType(Type),
            kString(Text),
            uiKeyCode(Code),
            ubWidth(Width),
            pkSubChars(SubChars)
        {}

        KeyboardKeyType_t eType; ///< type of key
        eC_String kString; ///< string to display
        eC_UInt uiKeyCode; ///< key-code
        eC_UByte ubWidth; ///< width in units
        CGUIRect kRect; ///< covering rect
        KeyboardSubChars_t* pkSubChars; ///< pointer to subchars
    };

    /// sub-chars
    struct KeyboardSubChars_t
    {
        /** default constructor
        */
        KeyboardSubChars_t() :
            uiNumRows(1)
        {}

        eC_UInt uiNumRows; ///< number of rows
        eC_TArray<KeyboardKey_t*> pkKeys; ///< keys
    };

    /// keyboard line
    struct KeyboardLine_t
    {
        /** default constructor
        */
        KeyboardLine_t() :
            uiTotalWidth(1)
        {}

        eC_UInt uiTotalWidth; ///< width in units
        CGUIRect kRect; ///< covering rect
        eC_TArray<KeyboardKey_t*> pkKeys; ///< keys
    };

    /// page-type
    enum KeyboardPageType_t
    {
        PT_UNKNOWN = 0,
        PT_LOWERCASE,
        PT_UPPERCASE,
        PT_NUMBERS,
        PT_SYMBOLS
    };

    /// page-struct
    struct KeyboardPage_t
    {
        /** default constructor
        */
        KeyboardPage_t() :
            eType(PT_UNKNOWN),
            ubIndex(0)
        {}

        KeyboardPageType_t eType; ///< type
        eC_UByte ubIndex; ///< index
        eC_TArray<KeyboardLine_t*> pkLines; ///< lines
    };

public:
    /** default constructor
    */
    CGUIOnscreenKeyboardLayout();

    /** destructor
    */
    virtual ~CGUIOnscreenKeyboardLayout();

    /** clear the layout
    */
    void Clear();

    /** calculate the rects for each line and key
    @param vWidth width
    @param vHeight height
    */
    void CalcKeyRects(const eC_Value& vWidth = eC_FromInt(0), const eC_Value& vHeight = eC_FromInt(0));

    /** get the number of pages
    @return number of pages
    */
    eC_UInt GetNumberOfPages() const;

    /** get the page for index
    @param uiIndex index
    @return page
    */
    KeyboardPage_t* GetPage(const eC_UInt& uiIndex);

    /** get the page including the given line
    @param pkLine line
    @return page
    */
    KeyboardPage_t* GetPage(KeyboardLine_t* pkLine);

    /** get the page including given key
    @param pkKey key
    @return page
    */
    KeyboardPage_t* GetPage(KeyboardKey_t* pkKey);

    /** find the page for the given type
    @param ePageType page-type
    @param ubIndex index of page
    @return true if page was found
    */
    eC_Bool FindPage(const KeyboardPageType_t& ePageType, eC_UByte& ubIndex);

    /** add a new page
    */
    void AddPage();

    /** add a new line
    @param pkPage page
    */
    void AddLine(KeyboardPage_t* pkPage);

    /** add a new key
    @param pkLine
    */
    void AddKey(KeyboardLine_t* pkLine);

    /** add a new subkey
    @param pkKey
    */
    void AddSubKey(KeyboardKey_t* pkKey);

    /** delete page
    @param pkPage
    @return true if successful
    */
    eC_Bool DeletePage(KeyboardPage_t* pkPage);

    /** delete line
    @param pkLine
    @return true if successful
    */
    eC_Bool DeleteLine(KeyboardLine_t* pkLine);

    /** delete key
    @param pkKey
    @return true if successful
    */
    eC_Bool DeleteKey(KeyboardKey_t* pkKey);

    /** delete subkey
    @param pkKey
    @return true if successful
    */
    eC_Bool DeleteSubKey(KeyboardKey_t* pkKey);

#ifdef GUILIANI_STREAM_GUI
    /** Reads all attributes from streaming file.
        This method is called by CGUIFactoryManager after one of the registered
        factories has created an instance of this class.
    */
    virtual void ReadFromStream();
#endif

#ifdef GUILIANI_WRITE_GUI
    /** Writes all attributes to the streaming file. A CGUIStreamWriter
        has to be initialized first.
        @param bWriteClassID This flag is used to select if writing of control
               class ID, leading and trailing tags is performed.
    */
    virtual void WriteToStream(const eC_Bool bWriteClassID = false);
#endif

private:
    eC_TArray<KeyboardPage_t*> m_pkPages; ///< pages
    eC_Value m_vWidth; ///< width
    eC_Value m_vHeight; ///< height
};

#endif
