/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GUIGESTUREDETECTOR_H__
#define __GUIGESTUREDETECTOR_H__

#include "eC_Types.h"
#include "GUIEvent.h"
#include "GUIPoint.h"

#define MAX_ACTIVE_POINTERS 3

/** this class defines an new event-class used for gestures
*/
class CGUIGestureEvent : public CGUIEvent
{
public:
    /** Constructor
    @param eType event-type
    @param iX x-position
    @param iY y-position
    @param vSpan diagonal span
    @param vSpanX span in x-direction
    @param vSpanY span in y-direction
    @param uiModifiers modifiers
    */
    CGUIGestureEvent(
        const EventType_t& eType,
        const eC_Int& iX,
        const eC_Int& iY,
        const eC_Value& vSpan,
        const eC_Value& vSpanX,
        const eC_Value& vSpanY,
        const eC_UInt& uiModifiers = 0) :
        CGUIEvent(eType, uiModifiers),
        m_iX(iX),
        m_iY(iY),
        m_vSpan(vSpan),
        m_vSpanX(vSpanX),
        m_vSpanY(vSpanY)
    {
    }

    /** Get x-position
    @return x
    */
    eC_Int GetX() const
    {
        return m_iX;
    }

    /** Get y-position
    @return y
    */
    eC_Int GetY() const
    {
        return m_iY;
    }

    /** Get diagonal span
    @return span
    */
    eC_Value GetSpan() const
    {
        return m_vSpan;
    }

    /** Get span in x-direction
    @return span-x
    */
    eC_Value GetSpanX() const
    {
        return m_vSpanX;
    }

    /** Get span in y-direction
    @return span-y
    */
    eC_Value GetSpanY() const
    {
        return m_vSpanY;
    }

private:
    eC_Int m_iX; ///< x-position
    eC_Int m_iY; ///< y-position
    eC_Value m_vSpan; ///< diagonal span
    eC_Value m_vSpanX; ///< span in x-direction
    eC_Value m_vSpanY; ///< span in y-direction
};

/** this class will create gesture events from multiple input-events */
class CGUIGestureDetector
{
public:
    /** event-types relevant for detecting gestures
    */
    enum GestureEventType_t
    {
        GET_PRIMARY_DOWN, ///< primary pointer down
        GET_PRIMARY_UP, ///< primary pointer up
        GET_ADDITIONAL_DOWN, ///< additional pointer down
        GET_ADDITIONAL_UP, ///< additional pointer up
        GET_MOVE ///< move of pointer
    };

    /** detected gesture-type
    */
    enum GestureType_t
    {
        GT_NONE, ///< none detected
        GT_SCALE, ///< scaling
        GT_SCROLL, ///< scrolling
        GT_ROTATE ///< rotation
    };

    /** pointer state
    */
    struct PointerState_t
    {
        /** default constructor to initialize pointer-state
        */
        PointerState_t() :
            m_bIsActive(false),
            m_bIsPrimary(false),
            m_uiPointerID(0),
            m_iX(0),
            m_iY(0)
        {
        }

        /** specific constructor
        @param bIsActive
        @param bIsPrimary
        @param uiPointerID
        @param iX
        @param iY
        */
        PointerState_t(
            const eC_Bool& bIsActive,
            const eC_Bool& bIsPrimary,
            const eC_UInt& uiPointerID,
            const eC_Int& iX,
            const eC_Int& iY) :
            m_bIsActive(bIsActive),
            m_bIsPrimary(bIsPrimary),
            m_uiPointerID(uiPointerID),
            m_iX(iX),
            m_iY(iY)
        {
        }

        eC_Bool m_bIsActive; ///< active-flag
        eC_Bool m_bIsPrimary; ///< true if primary
        eC_UInt m_uiPointerID; ///< pointer-id
        eC_Int m_iX; ///< x-position
        eC_Int m_iY; ///< y-position
    };

public:
    /** process a single input-event
    @param eType event-type
    @param kPointerState pointer-state
    @return valid event or NULL
    */
    static CGUIEvent* ProcessEvent(const GestureEventType_t& eType, const PointerState_t& kPointerState);

#ifdef GUILIANI_GESTURES_ACTIVE
private:
    /** update pointer data
    @param eType event-type
    @param kPointerState pointer-state
    @param iPointerIndex index of current pointer
    */
    static eC_Bool UpdatePointer(
        const GestureEventType_t& eType,
        const PointerState_t& kPointerState,
        eC_Int& iPointerIndex);

    /** clear all pointers
    */
    static void ClearPointers();
    static void Reset();

    static void UpdateScroll();
    static void UpdateScale();
    static void UpdateRotation();

private:
    static eC_UInt ms_uiActivePointers; ///< number of active pointers
    static PointerState_t ms_akPointers[MAX_ACTIVE_POINTERS]; ///< state of pointers

    static eC_Int ms_iSkipIndex;

    static CGUIPoint ms_kFocusPoint; ///< current position of focus point
    static CGUIPoint ms_kLastFocus; ///< last position of focus point

    static eC_Value ms_vCurrentSpanX; ///< current span in x-direction
    static eC_Value ms_vCurrentSpanY; ///< current span in y-direction
    static eC_Value ms_vCurrentSpan; ///< current diagonal span
    static eC_Value ms_vPreviousSpan; ///< previous diagonal span

    static eC_Value ms_vAverageDeviationX;
    static eC_Value ms_vAverageDeviationY;
    static eC_Value ms_vCurrentScrollX;
    static eC_Value ms_vCurrentScrollY;

    static eC_Value ms_vCurrentAngle;

    static eC_Int ms_iDownX; ///< position of down-event
    static eC_Int ms_iDownY; ///< position of down-event
    static GestureType_t ms_eCurrentType; ///< current gesture type
#endif
};

#endif
