/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef GUIIMAGEDATA__H_
#define GUIIMAGEDATA__H_

#include "eC_Types.h"
#include "GUICommonEnums.h"

// TODO: concept for supporting vector and bitmap-images
//new for SVGs
#include <vector>
#include "SVGShape.h"

/// Contains information about an image loaded by CGUIImageLoader.
/** Typical use case for this class is that a graphics wrapper creates an
    object of this class, sets some attributes to make sure the image uses the
    correct format etc., then passes it to the image loader which hands it over
    to the individual image decoders that fill the data section with the
    decoded image data.
  */
class CGUIImageData
{
public:
    /** the data-type
    */
    enum DataType_t
    {
        DT_BITMAP, ///< bitmap
        DT_VECTOR ///< vector
    };

    /** type of memory-usage
    */
    enum MemoryType_t
    {
        MT_MANAGED_INTERNAL, ///< internally managed, will be automatically freed at end of lifetime
        MT_MANAGED_EXTERNAL, ///< externally managed, will NOT be freed
        MT_ROM ///< ROM, will not be freed
    };

public:
    /** CGUIImageData constructor */
    CGUIImageData();

    /** CGUIImageData destructor */
    ~CGUIImageData();

    /** Operator= method. This does a flat copy of the given source CGUIImageData object, which means that both
        GUIImageData objects will then reference the SAME DATA in memory.
        @param kSource Source object to be copied.
        @return This instance.
    */
    CGUIImageData& operator=(const CGUIImageData& kSource);

    /** Sets the width of the image. Should only be called by an image decoder.
        @param uiWidth Width of the image.
      */
    void SetWidth(const eC_UInt &uiWidth);

    /** Sets the height of the image. Should only be called by an image decoder.
        @param uiHeight Height of the image.
      */
    void SetHeight(const eC_UInt &uiHeight);

    /** Sets the data pointer of the image. Should only be called by an image
        decoder.
        @param *pubData Pointer to memory to be used for storing a decoded image.
      */
    void SetData(eC_UByte *pubData);

    /** Sets the type of data (bitmap or vector)
        @param eType data-type
    */
    void SetDataType(const DataType_t& eType);

    /** Sets the memory-type for the image-data
        @param eType memory-type
    */
    void SetMemoryType(const MemoryType_t& eType);

    /** Sets the data-pointer of an SVG-image. Should only be called by an image.
    @param pkSVGShapes Pointer to std::vector<CSVGShape>.
    @todo calculate size and m_pubData has to be set but won't work with this approach.
    */
    void SetDataSVG(CSVGShape* pkSVGShapes);

    /** sets the number of shapes
    @param uiShapeCount
    */
    void SetShapeCount(const eC_UInt& uiShapeCount);

    /** Sets member to true if original image has alpha-channel
    @param bSourceAlpha
    */
    void SetSourceAlpha(const eC_Bool& bSourceAlpha);

    /** Sets the 'filler' value. It is typically used by decoders that load
        non-alpha images but store it in an alpha-enabled format. This
        'filler' is then used as the alpha value.
        This method should be called before CGUIImageDecoder::LoadImg().
        @param ubFillerByte Value used for the alpha filler.
      */
    void SetFiller(const eC_UByte &ubFillerByte);

    /** Sets the image format type.
        @param eImageType The format type.
      */
    void SetImageType(const CGUICommonEnums::ImageType_t &eImageType);

    /// @return The width of the image.
    eC_UInt GetWidth() const;

    /// @return The height of the image.
    eC_UInt GetHeight() const;

    /// @return Pointer to the decoded image data.
    eC_UByte* GetData() const;

    /// @return type of stored data
    DataType_t GetDataType() const;

    /// @return type of used memory
    MemoryType_t GetMemoryType() const;

    /** Gets the data pointer of an SVG-image.
    @return Pointer to CSVGShapes.
    @todo calculate size!
    */
    CSVGShape* GetDataSVG();

    /** Get the number of shapes
    @return number of shapes
    */
    eC_UInt GetShapeCount() const;

    /** @return Value of the 'filler' byte.
        For an explanation of this value, see SetFiller().
      */
    eC_UByte GetFiller() const;

    /// @return The format type of the image.
    CGUICommonEnums::ImageType_t GetImageType() const;

    /// @return The bit depth of the image.
    eC_UByte GetBytePerPixel() const;

    /** Returns the size (in bytes) of this image.
        Width, height and bits per pixel must have been set correctly before.
        @return Image size in bytes.
    */
    eC_UInt GetImageSize() const;

    /** Returns whether the image has alpha channel or not.
        @return True if the image has alpha data, otherwise False.
    */
    eC_Bool HasAlpha() const;

    /** Return whether the original image had an alpha-channel
        @return true if original image has alpha-channel
    */
    eC_Bool HasSourceAlpha() const;

    /** Deletes the image data array. */
    void DeleteData();

private:
    /** Copy-constructor. Do not use.
        Dummy declaration without implementation, just to hide it.
        @param kSource Source object to be copied.
    */
    CGUIImageData(const CGUIImageData& kSource);

private:
    eC_UInt m_uiWidth; ///< Width.
    eC_UInt m_uiHeight; ///< Height.
    CGUICommonEnums::ImageType_t m_eImageType; ///< loaded Format, may differ from original format

    eC_Bool m_bSourceAlpha; ///< this indicates if the source-image contains alpha channel

    /// Filler byte replaces the alpha value, if you load an image with alpha, but you don't want to use it. (used by BMP and JPEG)
    eC_UByte m_ubFillerByte;

    eC_UByte* m_pubData; ///< Image data.
    DataType_t m_eDataType;
    MemoryType_t m_eMemoryType;

    CSVGShape* m_pkSVGData;
    eC_UInt m_uiShapeCount;
};

#endif
