/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef GUI_TRANSITION_ENTER_LEAVE__H_
#define GUI_TRANSITION_ENTER_LEAVE__H_

#include "GUITransitionBase.h"
#include "GUIEasing.h"

/// Class to compute a transition from a source dialog to a destination dialog.
/** This class can move in the destination over the source or move out the source
    and reveal the destination underneath.
    Mode is selectable by animMode parameter.
    For basic transition behaviour see: @see CGUITransitionBase

    Example:
@code

    // Create a transition with a duration of 1000ms that lets the destination enter from left.
       pkTransition will delete itself after transition is finished.
    CGUITransitionEnterLeave* pkTransition = new CGUITransitionEnterLeave(1000, CGUITransitionEnterLeave::ENTER_FROM_LEFT, true);

    // Start the transition. The source dialog is deleted by the transition
       after completion.
    pkTransition->StartTransition(pSource, pDest, true);

@endcode
@ingroup GUILIANI_ANIMATIONS
*/
class CGUITransitionEnterLeave : public CGUITransitionBase
{
public:
    /** Used to specify the direction of push transition.
    */
    enum EnterLeaveMode_t
    {
        ENTER_FROM_LEFT,      ///< enter from left side of screen
        ENTER_FROM_RIGHT,     ///< enter from right side of screen
        ENTER_FROM_TOP,       ///< enter from top side of screen
        ENTER_FROM_BOTTOM,    ///< enter from bottom side of screen
        LEAVE_TO_LEFT,        ///< leave to left side of screen
        LEAVE_TO_RIGHT,       ///< leave to right side of screen
        LEAVE_TO_TOP,         ///< leave to top side of screen
        LEAVE_TO_BOTTOM       ///< leave to bottom side of screen
    };

    /// Constructs a EnterLeaveTransition.
    /**
        @param uiTransitionTime Transition time in ms.
        @param eMode The direction of push transition
        @param eEasingType The easing type of the push transition
        @param bAutoDelete The transition instance is deleted after transition has
        finished by default
    */
    CGUITransitionEnterLeave(
        const eC_UInt& uiTransitionTime,
        const EnterLeaveMode_t& eMode,
        const CGUIEasing::EasingType_t& eEasingType = CGUIEasing::EASE_LINEAR,
        const eC_Bool& bAutoDelete = true);
    ~CGUITransitionEnterLeave();

protected:
    void InitTransition();
    void EndTransition();
    void UpdateTransition(eC_Value vProgress);

private:
    EnterLeaveMode_t m_eMode;
    eC_Value m_vSrcPosX;
    eC_Value m_vSrcPosY;
    CGUIEasing::EasingType_t m_eEasingType;
};

#endif
