#ifndef __GUIBENCHMARK_H__
#define __GUIBENCHMARK_H__

#include "eC_TList_doubleLinked.h"

#include "GUI.h"
#include "GUIEvent.h"
#include "GUIInputMedia.h"
#include "GUIComponentManager.h"
//#include "GfxWrap.h"
#include "GUITrace.h"

#include "GUIResourceManager.h"
#include "GUIImageResource.h"


/**
This class provides a small stub for doing benchmarks.

it can benchmark primitives (e.g. rect, ellipse...) as well as blitting with different types.

which benchmark are done can be set via the SetXXX()-methods or AllOn() and AllOff()
the number of runs, primitives and blits which should be used can also be set.
images which should be used for the blitting can be added with AddTestImage()

to use this class simply create an instance and execute Run()

Note: to use images for benchmarks these need to be accessible by the CGUIResourceManager,
so either as files, within an resource-file or as a header-file compiled to the binary.
*/
class CGUIBenchmark
{
public:
    /** Constructor
    @param uiWidth width of the benchmark-area
    @param uiHeight height of the benchmark area
    @param uiStride stride of one line in bytes
    @param ubBitsPerPixel bit-depth
    @param pkFramebuffer pointer to framebuffers, if any
    */
    CGUIBenchmark(
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight,
        const eC_UInt& uiStride,
        const eC_UByte& ubBitsPerPixel,
        eC_UByte* pkFramebuffer);

    ~CGUIBenchmark();

    /** Run all configured benchmarks using all added images
    */
    void Run();

    /** Add an image to the tests
    @param kImageName name of the image
    */
    void AddTestImage(const eC_String& kImageName);

    /** Add a font to the tests
    @param kFontName name of the font
    */
    void AddTestFont(const eC_String& kFontName);

    /** Sets the font which is used for the summary
    @param kFontName
    */
    void SetFontForSummary(const eC_String& kFontName);

    /** Sets the number of seconds as the wait time for the summary
    @param uiWaitTime
    */
    void SetSummaryWaitTime(const eC_UInt& uiWaitTime);

    /** Set the number of runs
    @param uiNumberOfRuns
    */
    void SetNumberOfRuns(const eC_UInt& uiNumberOfRuns) { m_uiNumberOfRuns = uiNumberOfRuns; }

    /** Set the number primitives which should be used
    @param uiNumberOfPrimitives
    */
    void SetNumberOfPrimitives(const eC_UInt& uiNumberOfPrimitives) { m_uiNumberOfPrimitives = uiNumberOfPrimitives; }

    /** Set the number text which should be printed
    @param uiNumberOfTexts
    */
    void SetNumberOfTexts(const eC_UInt& uiNumberOfTexts) { m_uiNumberOfTexts = uiNumberOfTexts; }

    /** Set the number of blits which should be used
    @param uiNumberOfBlits
    */
    void SetNumberOfBlits(const eC_UInt& uiNumberOfBlits) { m_uiNumberOfBlits = uiNumberOfBlits; }

    /** set all benchmarks to active
    */
    void AllOn()
    {
        SetFillScreen(true);

        SetPrimitiveLine(true);
        SetPrimitiveRect(true);
        SetPrimitiveEllipse(true);
        SetPrimitiveArc(true);
        SetPrimitiveRing(true);

        SetTextSingle(true);
        SetTextMedium(true);
        SetTextLong(true);

        SetBlitImg(true);
        SetBlitImgNinePatch(true);
        SetBlitImgExt(true);
        SetBlitSubImage(true);
        SetRotate(true);
        SetRotateSubImage(true);
    }

    /** set all benchmarks to inactive
    */
    void AllOff()
    {
        SetFillScreen(false);

        SetPrimitiveLine(false);
        SetPrimitiveRect(false);
        SetPrimitiveEllipse(false);
        SetPrimitiveArc(false);
        SetPrimitiveRing(false);

        SetTextSingle(false);
        SetTextMedium(false);
        SetTextLong(false);

        SetBlitImg(false);
        SetBlitImgNinePatch(false);
        SetBlitImgExt(false);
        SetBlitSubImage(false);
        SetRotate(false);
        SetRotateSubImage(false);
    }

    /** activate or de-activate benchmark for screen-filling
    @param bActive if true benchmark will be executed
    */
    void SetFillScreen(const eC_Bool& bActive) { m_bFillScreenActive = bActive; }

    /** activate or de-activate benchmark for lines
    @param bActive if true benchmark will be executed
    */
    void SetPrimitiveLine(const eC_Bool& bActive) { m_bPrimitiveLineActive = bActive; }
    /** activate or de-activate benchmark for rectangles
    @param bActive if true benchmark will be executed
    */
    void SetPrimitiveRect(const eC_Bool& bActive) { m_bPrimitiveRectActive = bActive; }
    /** activate or de-activate benchmark for ellipses
    @param bActive if true benchmark will be executed
    */
    void SetPrimitiveEllipse(const eC_Bool& bActive) { m_bPrimitiveEllipseActive = bActive; }
    /** activate or de-activate benchmark for arcs
    @param bActive if true benchmark will be executed
    */
    void SetPrimitiveArc(const eC_Bool& bActive) { m_bPrimitiveArcActive = bActive; }
    /** activate or de-activate benchmark for rings
    @param bActive if true benchmark will be executed
    */
    void SetPrimitiveRing(const eC_Bool& bActive) { m_bPrimitiveRingActive = bActive; }

    /** activate or de-activate benchmark for text with single characters
    @param bActive if true benchmark will be executed
    */
    void SetTextSingle(const eC_Bool& bActive) { m_bTextSingleActive = bActive; }
    /** activate or de-activate benchmark for text with medium length (10 - 50)
    @param bActive if true benchmark will be executed
    */
    void SetTextMedium(const eC_Bool& bActive) { m_bTextMediumActive = bActive; }
    /** activate or de-activate benchmark for text with great length (50 - 200)
    @param bActive if true benchmark will be executed
    */
    void SetTextLong(const eC_Bool& bActive) { m_bTextLongActive = bActive; }

    /** activate or de-activate benchmark for simple blit
    @param bActive if true benchmark will be executed
    */
    void SetBlitImg(const eC_Bool& bActive) { m_bBlitImgActive = bActive; }
    /** activate or de-activate benchmark for blit using nine-patches
    @param bActive if true benchmark will be executed
    */
    void SetBlitImgNinePatch(const eC_Bool& bActive) { m_bBlitImgNinePatchActive = bActive; }
    /** activate or de-activate benchmark for extended blit
    @param bActive if true benchmark will be executed
    */
    void SetBlitImgExt(const eC_Bool& bActive) { m_bBlitImgExtActive = bActive; }
    /** activate or de-activate benchmark for blit of sub-image
    @param bActive if true benchmark will be executed
    */
    void SetBlitSubImage(const eC_Bool& bActive) { m_bBlitSubImageActive = bActive; }
    /** activate or de-activate benchmark for rotated blit
    @param bActive if true benchmark will be executed
    */
    void SetRotate(const eC_Bool& bActive) { m_bRotateActive = bActive; }
    /** activate or de-activate benchmark for rotated blit with sub-image
    @param bActive if true benchmark will be executed
    */
    void SetRotateSubImage(const eC_Bool& bActive) { m_bRotateSubImageActive = bActive; }

    // helper classes
private:
    /** small stub as an inputmedia to deliver events
    this will always return NULL as event
    */
    class InputHelper : public CGUIInputMedia
    {
    public:
        /** Constructor
        */
        InputHelper();

        /** retrieve an event
        @param uiIdleTime will be ignored
        @return always NULL
        */
        virtual void PollEvent();
    };

    /**
    This is a special GUI which can force a refresh of the screen
    */
    class GUIHelper : public CGUI
    {
    public:
        /** Constructor
        @param uiWidth width
        @param uiHeight height
        */
        GUIHelper(const eC_UInt& uiWidth, const eC_UInt& uiHeight);

        /** Refresh the complete screen
        */
        void Refresh();

        /** Set the foreground-color as 32bit
        @param uiColor
        */
        void SetColor(const eC_UInt& uiColor = 0);

        /** Set the foreground-color with RGBA
        @param ubRed
        @param ubGreen
        @param ubBlue
        @param ubAlpha
        */
        void SetColor(const eC_UByte& ubRed, const eC_UByte& ubGreen, const eC_UByte& ubBlue, const eC_UByte& ubAlpha);

        /** Clear the screen with 32bit
        @param uiColor
        */
        void Clear(const eC_UInt& uiColor = 0);

        /** Clear the screen with RGBA
        @param ubRed
        @param ubGreen
        @param ubBlue
        @param ubAlpha
        */
        void Clear(const eC_UByte& ubRed, const eC_UByte& ubGreen, const eC_UByte& ubBlue, const eC_UByte& ubAlpha);

        virtual eC_UInt GetIdleTime() const
        {
            return 0;
        }

        void StartDraw()
        {
            m_tDrawTime = eC_GetTicks64();
        }

        void EndDraw()
        {
            m_uiDrawTime += eC_GetMicroSec(eC_GetTicks64() - m_tDrawTime);
        }

        void ResetTimer()
        {
            m_uiDrawTime = 0;
            m_uiRefreshTime = 0;
        }

        void GetTime(eC_Value& vDrawTime, eC_Value& vRefreshTime)
        {
            vDrawTime = eC_Div(eC_FromInt(m_uiDrawTime), eC_FromInt(1000));
            vRefreshTime = eC_FromInt(m_uiRefreshTime);
        }

    private:
        eC_UInt m_uiWidth;
        eC_UInt m_uiHeight;

        eC_Time64 m_tDrawTime;
        eC_UInt m_uiDrawTime;
        eC_UInt m_uiRefreshTime;
    };

private:
    // no copy constructor
    CGUIBenchmark(const CGUIBenchmark&);

    /** Execute benchmark for filling the wohle screen
    needs access to framebuffer
    */
    eC_UInt BenchmarkFillScreen();

    /** Execute benchmark for line-primitives
    */
    eC_UInt BenchmarkLine();

    /** Execute benchmark for rectangle-primitives
    @param bFilled true if filled
    */
    eC_UInt BenchmarkRect(const eC_Bool& bFilled);

    /** Execute benchmark for ellipse-primitives
    @param bFilled true if filled
    */
    eC_UInt BenchmarkEllipse(const eC_Bool& bFilled);

    /** Execute benchmark for arc-primitives
    @param bFilled true if filled
    */
    eC_UInt BenchmarkArc(const eC_Bool& bFilled);

    /** Execute benchmark for ring-primitives
    @param bFilled true if filled
    */
    eC_UInt BenchmarkRing(const eC_Bool& bFilled);

    // text
    /** Execute benchmark for text with single characters
    */
    eC_UInt BenchmarkText_Single();

    /** Execute benchmark for text with medium length
    */
    eC_UInt BenchmarkText_Medium();

    /** Execute benchmark for text with great length
    */
    eC_UInt BenchmarkText_Long();

    /** benchmark of
    BlitImgExt with ID, dstX and dstY
    */
    eC_UInt BenchmarkBlitImg_IDXY();

    /** benchmark of
    BlitImgExt with ID, dstX, dstY, width, height, stretch and Alpha
    */
    eC_UInt BenchmarkBlitImgExt_IDXYWHSA(const eC_Value& vFactor, const eC_UByte& ubAlpha);

    /** benchmark of
    BlitImgNinePatch with ID, dstRect, ninePatch and Alpha
    */
    eC_UInt BenchmarkBlitImgNinePatch_IDRNA(
        const eC_Value& vFactorX,
        const eC_Value& vFactorY,
        const eC_UByte& ubAlpha);

    /** benchmark of
    BlitImgExt with ID, srcRect, dstRect and Alpha
    */
    eC_UInt BenchmarkBlitImgExt_IDRRA(const eC_Value& vFactor, const eC_UByte& ubAlpha);

    /** benchmark of
    BlitImgExt with ID, srcX, srcY, srcWidth, srcHeight, dstX, dstY, dstWidth, dstHeight, alpha and angle
    */
    eC_UInt BenchmarkBlitImgExt_IDXYWHXYWHAAN(
        const eC_UInt uiSrcX,
        const eC_UInt uiSrcY,
        const eC_UInt uiSrcWidth,
        const eC_UInt uiSrcHeight,
        const eC_Value& vFactor,
        const eC_UByte& ubAlpha);

    /** sets the currently used image for benchmarking
    @param kImageName filename of the image
    @return true if image was successfully loaded
    */
    eC_Bool SetTestImage(const eC_String& kImageName);

    /** sets the currently used font for benchmarking
    @param kFontName filename of the font
    @param uiFontSize size of the font (does not work for bitmap-fonts)
    @return true if font was successfully loaded
    */
    eC_Bool SetTestFont(
        const eC_String& kFontName,
        const eC_UInt& uiFontSize);

    /** Start the test
    will get the current system-time
    @return true if successful
    */
    eC_Bool StartTest();

    /** End the test
    @return number of millisecond for the execution
    */
    eC_UInt EndTest();

    /** Get the number of millisecond of the system-time
    @return number of milliseconds
    */
    eC_UInt GetTime();

    eC_Value CalcTextScore();
    eC_Value CalcBlitScore();

    /** output summary of test-runs
    */
    void PrintPrimitivesSummary();
    void PrintTextSummary();
    void PrintBlitSummary();
    void PrintScores();

    void PrintSummaryLine(const eC_String& kText, eC_Value& vNextLine, eC_Value& vNextColumn);

    inline eC_Value GetXPos(const eC_UInt& uiRuns, const eC_UInt& uiActions)
    {
        // have -50 to 50 variance
        if (uiActions % 2)
            return eC_FromInt(((uiRuns + 10) * uiActions * ((m_uiNumberOfRuns + 5) * 2 - uiActions)) % (m_uiWidth + 100) - eC_FromFloat(50.3f));
        else
            return eC_FromInt(((uiRuns + 20) * uiActions * ((m_uiNumberOfRuns + 7) * 2 - uiActions)) % (m_uiWidth + 100) - eC_FromFloat(49.7f));
    }

    inline eC_Value GetYPos(const eC_UInt& uiRuns, const eC_UInt& uiActions)
    {
        // have -50 to 50 variance
        if (uiActions % 2)
            return eC_FromInt(((uiRuns + 20) * uiActions * ((m_uiNumberOfRuns + 5) / 2 - uiActions)) % (m_uiHeight + 100) - eC_FromFloat(49.7f));
        else
            return eC_FromInt(((uiRuns + 10) * uiActions * ((m_uiNumberOfRuns + 7) / 2 - uiActions)) % (m_uiHeight + 100) - eC_FromFloat(50.3f));
    }

    inline void GetX2Pos(const eC_UInt& uiRuns, const eC_UInt& uiActions, eC_Value& vX1, eC_Value& vX2)
    {
        if (uiActions % 2)
            vX1 = eC_FromInt(((uiRuns + 10) * uiActions * ((m_uiNumberOfRuns + 5) * 3 - uiActions)) % (m_uiWidth + 100) - eC_FromFloat(50.3f));
        else
            vX1 = eC_FromInt(((uiRuns + 20) * uiActions * ((m_uiNumberOfRuns + 7) * 3 - uiActions)) % (m_uiWidth + 100) - eC_FromFloat(49.7f));
        vX2 = vX1 + eC_Abs(eC_FromInt(((uiRuns + 10) * uiActions * ((m_uiNumberOfRuns + 5) * 3 - uiActions)) % (m_uiWidth + 100)));
    }

    inline void GetY2Pos(const eC_UInt& uiRuns, const eC_UInt& uiActions, eC_Value& vY1, eC_Value& vY2)
    {
        if (uiActions % 2)
            vY1 = eC_FromInt(((uiRuns + 20) * uiActions * ((m_uiNumberOfRuns + 5) / 3 - uiActions)) % (m_uiHeight + 100) - eC_FromFloat(49.7f));
        else
            vY1 = eC_FromInt(((uiRuns + 10) * uiActions * ((m_uiNumberOfRuns + 7) / 3 - uiActions)) % (m_uiHeight + 100) - eC_FromFloat(50.3f));
        vY2 = vY1 + eC_Abs(eC_FromInt(((uiRuns + 20) * uiActions * ((m_uiNumberOfRuns + 5) / 3 - uiActions)) % (m_uiWidth + 100)));
    }

private:
    struct TimeData_t
    {
        TimeData_t() :
            DataPresent(false),
            Name(""),
            TimePerRun(eC_FromInt(0)),
            OperationsPerSecond(0),
            DrawTime(eC_FromInt(0)),
            RefreshTime(eC_FromInt(0)),
            HasAdditionalData(false),
            FactorX(eC_FromInt(0)),
            FactorY(eC_FromInt(0)),
            Alpha(0)
        {}

        TimeData_t(
            const eC_String& kName,
            const eC_Value& vTimePerRun,
            const eC_UInt& uiOperationsPerSecond,
            const eC_Value& vDrawTime,
            const eC_Value& vRefreshTime,
            const eC_Bool& bHasAdditionalData = false,
            const eC_Value& vFactorX = eC_FromInt(0),
            const eC_Value& vFactorY = eC_FromInt(0),
            const eC_UByte& ubAlpha = 0) :
            DataPresent(true),
            Name(kName),
            TimePerRun(vTimePerRun),
            OperationsPerSecond(uiOperationsPerSecond),
            DrawTime(vDrawTime),
            RefreshTime(vRefreshTime),
            HasAdditionalData(bHasAdditionalData),
            FactorX(vFactorX),
            FactorY(vFactorY),
            Alpha(ubAlpha)
        {}

        eC_Bool DataPresent;
        eC_String Name;
        eC_Value TimePerRun;
        eC_UInt OperationsPerSecond;
        eC_Value DrawTime;
        eC_Value RefreshTime;
        eC_Bool HasAdditionalData;
        eC_Value FactorX;
        eC_Value FactorY;
        eC_UByte Alpha;
    };

private:
    // sizes and framebuffer access
    eC_UInt m_uiWidth;
    eC_UInt m_uiHeight;
    eC_UInt m_uiStride;
    eC_UByte m_ubBitsPerPixel;
    eC_UByte* m_pkFrameBuffer;

    // helpers
    eC_UInt m_uiNumberOfRuns;
    eC_UInt m_uiNumberOfPrimitives;
    eC_UInt m_uiNumberOfTexts;
    eC_UInt m_uiNumberOfBlits;
    GUIHelper* m_pkGUI;
    CGfxWrap* m_pkGfxWrap;
    InputHelper* m_pkInput;

    // image
    eC_TListDoubleLinked<eC_String*> m_kImageList;
    ImageResource_t m_eImageID;
    eC_UInt m_uiImageWidth;
    eC_UInt m_uiImageHeight;

    // font
    eC_TListDoubleLinked<eC_String*> m_kFontList;
    FontResource_t m_eFontID;

    // summary
    eC_String m_kSummaryFontName;
    FontResource_t m_eSummaryFontID;
    eC_UInt m_uiWaitTime;

    // time
    eC_Time64 m_tTime;
    eC_UInt m_uiStartMS;
    eC_UInt m_uiEndMS;

    eC_Value m_vRedrawTime;
    eC_Value m_vRefreshTime;

    // test-flags

    // fill screen
    eC_Bool m_bFillScreenActive;
    TimeData_t m_kFillScreenData;

    // primitives
    eC_Bool m_bPrimitiveLineActive;
    TimeData_t m_kLineData;
    eC_Bool m_bPrimitiveRectActive;
    TimeData_t m_kRectData[2];
    eC_Bool m_bPrimitiveEllipseActive;
    TimeData_t m_kEllipseData[2];
    eC_Bool m_bPrimitiveArcActive;
    TimeData_t m_kArcData[2];
    eC_Bool m_bPrimitiveRingActive;
    TimeData_t m_kRingData[2];

    // texts
    eC_Bool m_bTextSingleActive;
    TimeData_t m_kTextSingleData;
    eC_Bool m_bTextMediumActive;
    TimeData_t m_kTextMediumData;
    eC_Bool m_bTextLongActive;
    TimeData_t m_kTextLongData;
    eC_Value m_vTextScore;

    // blits
    eC_Bool m_bBlitImgActive;
    TimeData_t m_kBlitImgData;
    eC_Bool m_bBlitImgNinePatchActive;
    TimeData_t m_kBlitImgNinePatchData[10];
    eC_Bool m_bBlitImgExtActive;
    TimeData_t m_kBlitImgExtData[4];
    eC_Bool m_bBlitSubImageActive;
    TimeData_t m_kBlitSubImageData[4];
    eC_Bool m_bRotateActive;
    TimeData_t m_kRotateData;
    eC_Bool m_bRotateSubImageActive;
    TimeData_t m_kRotateSubImageData;
    eC_Value m_vBlitScore;

    eC_Value m_vNextLine;
    eC_Value m_vNextColumn;
};

#endif
