/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GUIMEMPROFILE_H__
#define __GUIMEMPROFILE_H__

#include "GUIAnimatable.h"
#include "GUICompositeObject.h"

#include "GUIFontResource.h"
#include "GUIRect.h"
#include "eC_String.h"
#include "eC_TList_doubleLinked.h"

#include "GUIComponentManager.h"

#define GETPROFILER CGUIComponentManager::GetInstance().GetProfiler()

/**

This class is used for profiling purposes in Guiliani.

You can measure the following things during runtime:

- CPU

The CPU value is calculated by taking two different time-spans (processing time / complete time for one loop)
The first one is the time which internal processing in Guiliani takes up.
This one does not include acquiring new events from the InputDevice.
This is done because, when there is no event the InputDevice will wait for a timeout.
The second value is the time which elapses for the complete run.

So the term CPU-consumption might not be the correct one, since in a multi-tasking environment the scheduler
might split one loop of Guiliani into several small parts, which are interrupted by other tasks.
It is more like a percentage of the whole timeslice Guiliani has had to the part of the timeslice for processing.
So 50% means that 50% of the time inside the Guiliani main-loop was spent for the input-device and 50% for processing,
whereas 100% means that there was nearly no time consumed by waiting for events.

- FPS

There are two different values for the FPS of Guiliani:
1. FPS measures only the time for redrawing the invalidated regions on the screen.
This value may get really high if there is nothing to be drawn on the screen.

2. AbsFPS measures the time for a whole cycle through the main-loop of Guiliani,
including processing time of internal stuff (Commands, DataPool, etc.)
This value is comparable to the FPS-value you are used from games or other applications.

- Memory

There a several measurements for memory-profiling available:
1. used memory
the number of bytes currently used by allocation in Guiliani

2. maximum value for allocated memory
3. largest block ever allocated
these two values can be reset during runtime for analyzing different situations

4. number of allocated blocks

Note: please keep in mind that only memory which is allocated by new/delete is counted.
if you use the C-functions malloc/free the memory will not be tracked by the profiler.

It is also possible to display a bar of the consumed memory on the screen, where the color of the bar
indicated if a threshold (which can be set seperately) was reached.
The length of the bar is the total amount of memory available.
The colored bar (green, yellow, red) indicates the currently used memory.
The blue bar in the background indicates the maximum amount of allocated memory.

- Graphics

for measuring graphics performance in various situations the duration of redraw and refresh
are calculated and can be displayed during runtime.

*/
class CGUIProfiler : public CGUIAnimatable
{
public:
    friend class CGUI; ///< for private access
    friend class CGfxWrap; ///< for private access
    friend class CGUIComponentManager; ///< for private access

public:
    /** Enumeration for the counter IDs*/
    enum FPSCounterID
    {
        INVALID_COUNTER_ID = -1,
        DEFAULT_FPS,
        DEFAULT_ABS_FPS
    };

public:
    /** Create Instance
    */
    static void CreateInstance();

    /** Delete Instance
    */
    static void DeleteInstance();

    // output methods

    /** Set profiler to active.
    @param bActive
    */
    void SetActive(const eC_Bool& bActive);

    /** get the active-state of the profiler
    @return true if active
    */
    eC_Bool IsActive() const
    {
        return m_bActive;
    }

    /** Set logging of profiling-data on the screen to active
    @param bActive
    */
    void SetDisplayActive(const eC_Bool& bActive);

    /** return if the onscreen-display is currently active
    @return true if active
    */
    eC_Bool IsDisplayActive() const
    {
        return m_bDisplayActive;
    }

    /** Set logging of profiling-data to the console to active
    @param bActive
    */
    void SetConsoleActive(const eC_Bool& bActive);

    /** return if the console-output is currently active
    @return true if active
    */
    eC_Bool IsConsoleActive() const
    {
        return m_bConsoleActive;
    }

    // profiling

    /** Set profiling of CPU
    @param bActive
    */
    void SetCPUProfilingActive(const eC_Bool& bActive);

    /** return if CPU-profiling is currently active
    @return true if active
    */
    eC_Bool IsCPUProfilingActive() const
    {
        return m_bCPUProfilingActive;
    }

    /** Set profiling of memory-usage
    @param bActive
    */
    void SetMemoryProfilingActive(const eC_Bool& bActive);

    /** return if the memory-profiling is currently active
    @return true if active
    */
    eC_Bool IsMemoryProfilingActive() const
    {
        return m_bMemoryProfilingActive;
    }

    /** Set display of current memory-usage as a bar
    @param bActive
    */
    void SetMemoryUsageBarActive(const eC_Bool& bActive);

    /** return if the memory-usage-bar is currently displayed
    @return true if active
    */
    eC_Bool IsMemoryUsageBarActive() const
    {
        return m_bMemoryUsageBarActive;
    }

    /** Set profiling of graphics
    @param bActive
    */
    void SetGraphicsProfilingActive(const eC_Bool& bActive);

    /** return if graphics-profiling is currently active
    @return true if active
    */
    eC_Bool IsGraphicsProfilingActive() const
    {
        return m_bGraphicsProfilingActive;
    }

    /** Set display of invalidation
    @param bActive
    */
    void SetInvalidationActive(const eC_Bool& bActive);

    /** return if invalidation is currently active
    @return true if active
    */
    eC_Bool IsInvalidationActive() const
    {
        return m_bInvalidationActive;
    }

    /** Set display of FPS-counters
    @param bActive
    */
    void SetFPSActive(const eC_Bool& bActive);

    /** return if FPS-measuring is currently active
    @return true if active
    */
    eC_Bool IsFPSActive() const
    {
        return m_bFPSActive;
    }

    /** Set display of runtime
    @param bActive
    */
    void SetRuntimeActive(const eC_Bool& bActive);

    /** return if Runtime is active
    @return true if active
    */
    eC_Bool GetRuntimeActive() const
    {
        return m_bRuntimeActive;
    }

    /** Set display of object count
    @param bActive
    */
    void SetObjectCountActive(const eC_Bool& bActive);

    /** return if object count is active
    @return true if active
    */
    eC_Bool GetObjectCountActive() const
    {
        return m_bObjectCountActive;
    }

    // memory-settings

    /** Set the amount of available memory
    @param tMemoryAvailable number of bytes available
    */
    void SetMemoryAvailable(const size_t& tMemoryAvailable);

    /** Get the amount of available memory in bytes
    @return available memory
    */
    size_t GetMemoryAvailable() const;

    /** set thresholds for memory usage
    @param uiMemoryUsageWarning warning-level
    @param uiMemoryUsageAlert alert-level
    */
    void SetMemoryThresholds(const size_t& uiMemoryUsageWarning, const size_t& uiMemoryUsageAlert);

    /** get thresholds for memory usage
    @param uiMemoryUsageWarning warning-level
    @param uiMemoryUsageAlert alert-level
    */
    void GetMemoryThresholds(size_t& uiMemoryUsageWarning, size_t& uiMemoryUsageAlert);

    // general settings

    /** Set the updating interval
    @param uiUpdateInterval time in milliseconds
    */
    void SetUpdateInterval(const eC_UInt& uiUpdateInterval);

    /** Set the background-color
    @param uiBackColor color in ARGB
    */
    void SetBackColor(const eC_UInt& uiBackColor);

    /** Set the text-color
    @param uiTextColor color in ARGB
    */
    void SetTextColor(const eC_UInt& uiTextColor);

    /** Set the font-id
    @param eFontID used font
    */
    void SetFont(const FontResource_t& eFontID);

    /** Set the position for the output
    @param vX x-position
    @param vY y-position
    */
    void SetPosition(const eC_Value& vX, const eC_Value& vY);

    /** Set the position for the memory-bar
    @param vX x-position
    @param vY y-position
    */
    void SetMemoryUsageBarPosition(const eC_Value& vX, const eC_Value& vY);

    /** Set the size for the memory-bar
    @param vWidth width
    @param vHeight height
    */
    void SetMemoryUsageBarSize(const eC_Value& vWidth, const eC_Value& vHeight);

    // getter for values

    /** get the current CPU-Usage
    @return CPU-usage
    */
    eC_UInt GetCPUUsage() const;

    /** Get amount of used memory
    @return used memory in bytes
    */
    size_t GetMemoryUsed() const;

    /** Get the maximum amount of memory used
    @return max in bytes
    */
    size_t GetMemoryMax() const;

    /** Clear the value for maximum allocated memory
    */
    void ClearMemoryMax();

    /** Get the largest block ever allocated
    @return size of largest block in bytes
    */
    size_t GetMemoryLargestBlock() const;

    /** Clear the value for the largest allocated block of memory
    */
    void ClearMemoryLargestBlock();

    /** Get the duration of the last redraw
    @return redraw-duration in ms
    */
    eC_UInt GetRedrawDuration() const;

    /** Get the duration of the last refresh
    @return refresh-duration in ms
    */
    eC_UInt GetRefreshDuration() const;

    /** return the number of object used in the currently displayed GUI
    @return number of objects
    */
    eC_UInt GetObjectCount();

    virtual void DoAnimate(const eC_Value& vTimes = eC_FromInt(1));

private:
    /** Begin the main-loop
    */
    void BeginLoop();

    /** Begin the process in main-loop
    after the Input-Device is queried
    */
    void BeginProcess();

    /** End the main-loop
    */
    void EndLoop();

    /** Start a new measurement for FPS
    */
    void StartMeasurementFPS();

    /** End the measurement for FPS
    */
    void EndMeasurementFPS();

    /** Start a new redraw-phase
    */
    void StartRedraw();

    /** End the current redraw-phase
    */
    void EndRedraw();

    /** Start a new refresh-phase
    */
    void StartRefresh();

    /** End the current refresh-phase
    */
    void EndRefresh();

    /** Display a new invalidation rect
    @param kRect rect of invalidation
    */
    void DisplayInvalidation(const CGUIRect& kRect);

    /** return the number of object used in the currently displayed GUI
    @param pkContainer container to count objects in
    @param ruiCount object count
    */
    void GetObjectCount(CGUICompositeObject* pkContainer, eC_UInt& ruiCount);

private:
    /**stanadrd CGUIFramerate constructor
    @see SetPosition*/
    CGUIProfiler();

    /** CGUIFramerate destructor*/
    virtual ~CGUIProfiler();

    /** calculates the current FPS for all counters*/
    void Update();

    /** draws all visible FPS counters on the screen*/
    void Draw() const;

    void DrawMemoryBar() const;

    /** log output to console */
    void ConsoleOutput() const;

    void CalcBackRectangle();

    /** Adds a counter.
    @param kFPSTextPrefix Text prefix to display.
    @return id of the new counter
    */
    FPSCounterID AddCounter(const eC_String& kFPSTextPrefix);

    /** Get the frames per second of the default FPS counter.
    @param eCounterID id of the counter
    @return frames per second for the given counter id*/
    eC_UInt GetFPS(const FPSCounterID& eCounterID = DEFAULT_FPS) const;

    /** Start the measurement for the FPS.
    @param eCounterID ID of the counter.
    */
    void StartMeasurement(const FPSCounterID& eCounterID) const;

    /** Stop the measurement for the FPS.
    @param eCounterID ID of the counter.
    */
    void StopMeasurement(const FPSCounterID& eCounterID) const;

private:
    /**
    Class for generating counters which are administrated by the GUIFramerate class.
    */
    class CGUIFramerateCounter
    {
    public:
        /** Standard CGUIFramerateCounter constructor*/
        CGUIFramerateCounter();

        /** CGUIFramerateCounter constructor with given FPS text
            the given text is added to FPS number
            i.e. given text is "overall FPS: " the output on the screen
            would be "overall FPS: 78.5"
            @param kFPSTextPrefix Text prefix to display.
        */
        CGUIFramerateCounter(eC_String kFPSTextPrefix);

        /** CGUIFramerateCounter destructor*/
        virtual ~CGUIFramerateCounter();

        /** Get the last calculated FPS rate
            @return last calculated FPS rate
            */
        eC_UInt GetFPS() const;

        /** Return the current text of the counter which
        includes the prefix and current value
        @return text of counter
        */
        eC_String GetText() const;

        /** Inform the FPS counter about a new frame */
        void CountFrame();

        /** Start the measurement for this FPS */
        void StartMeasurement();

        /** Stop the measurement for this FPS */
        void StopMeasurement();

        /** Recalculate the FPS*/
        void Recalculate();

    public:
        eC_UInt m_uiFrames; ///< number of frames
        eC_Time64 m_kStartTime; ///< start time of measurement
        eC_Time64 m_kDeltaTime; ///< measured time since the last recalculation
        eC_UInt m_uiFPS; /// frames per second (FPS)
        eC_String m_kFPSTextPrefix; ///< FPS text prefix
        eC_String m_kFPSText; ///< FPS text
    };

private:
    /** Inform the FPS counter about a new frame.
    @param eCounterID ID of the counter.
    */
    void CountFrame(const FPSCounterID& eCounterID) const;

    /** calculates the current FPS for all counters*/
    void Calculate();

    /** Access a counter with given ID.
    @param eCounterID ID of teh the counter to access
    @return Pointer to the counter withe given ID, if there is no counter with
    given ID, a NULL pointer is returned.*/
    CGUIFramerateCounter* GetCounter(const FPSCounterID& eCounterID) const;

    void CheckUpdateTimer();

    void UpdateMemoryValues();

    void UpdateObjectCount();

private:
    eC_Bool m_bActive;
    eC_Bool m_bDisplayActive;
    eC_Bool m_bConsoleActive;
    eC_Bool m_bCPUProfilingActive;
    eC_Bool m_bMemoryProfilingActive;
    eC_Bool m_bMemoryUsageBarActive;
    eC_Bool m_bGraphicsProfilingActive;
    eC_Bool m_bInvalidationActive;
    eC_Bool m_bFPSActive;
    eC_Bool m_bRuntimeActive;
    eC_Bool m_bObjectCountActive;

    eC_UInt m_uiUpdateInterval;

    eC_UInt m_uiBackColor;
    eC_UInt m_uiTextColor;
    FontResource_t m_eFontID;

    eC_Value m_vPosX;
    eC_Value m_vPosY;
    eC_UInt m_uiLineCount;
    eC_Value m_vLineHeight;

    eC_Value m_vMemoryBarPosX;
    eC_Value m_vMemoryBarPosY;
    eC_Value m_vMemoryBarWidth;
    eC_Value m_vMemoryBarHeight;

    CGUIRect m_kBackRect;

    eC_Time64 m_tLoopBegin;
    eC_Time64 m_tProcessBegin;

    eC_UInt m_uiCPUUsage;
    eC_UInt m_uiLastCPUUsage;
    size_t m_tMemoryUsed;
    size_t m_tMemoryUsageWarning;
    size_t m_tMemoryUsageAlert;
    size_t m_tMemoryUsedMax;
    size_t m_tMemoryUsedLargestBlock;
    size_t m_tMemoryNumAllocs;
    eC_UInt m_uiStartRedraw;
    eC_UInt m_uiLastRedraw;
    eC_UInt m_uiDurationRedraw;
    eC_UInt m_uiStartRefresh;
    eC_UInt m_uiLastRefresh;
    eC_UInt m_uiDurationRefresh;

    eC_String m_kCPUUsage;
    eC_String m_kMemoryUsed;
    eC_String m_kMemoryMax;
    eC_String m_kMemoryLargestBlock;
    eC_String m_kMemoryNumAllocs;
    eC_String m_kTimeRedraw;
    eC_String m_kTimeRefresh;
    eC_String m_kTimeRuntime;
    eC_String m_kObjectCount;

    eC_Value m_vScreenWidth;
    eC_Value m_vScreenHeight;
    eC_Value m_vMemoryUsageCurrent;
    eC_Value m_vMemoryUsageWarning;
    eC_Value m_vMemoryUsageAlert;
    eC_Value m_vMemoryUsageMax;

    eC_UInt m_uiObjectCount;

    eC_Bool m_bNeedsRedraw;
    eC_Bool m_bRectChanged;

    eC_Time64 m_tStartTime;

    // FPS
    /// internal time
    eC_Time64 m_kInternalTimer;

    /// list of framerate counters
    eC_TListDoubleLinked<CGUIFramerateCounter*> m_kCounterList;

    /// ID for the next counter to add
    FPSCounterID m_eLastCounterID;
};

#endif
