/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

/** @brief Contains declarations for functions that are called during start-up
           for obtaining custom extensions.
  */
#ifndef __CUSTOM_EXTENSION_FUNCS_H__
#define __CUSTOM_EXTENSION_FUNCS_H__

#include "CustomExtensionFactory.h"

#include "GUICommonEnums.h"

#include <vector>

/** Describes an enumeration value so that it can be mapped to readable text in
    the editor's GUI. */
struct EnumMapping
{
    /** Constructs an enumeration mapping descriptor.
        @param rkTypeTag The XML tag used when reading from or writing the
               enumeration value to a stream. In other words, this must be the
               same string that is used in the respective control's/command's/
               behaviour's/layouter's WriteToStream and ReadFromStream
               implementations.
        @param iEnumValue The enumeration value.
        @param rkValueName The string representation to be used for iEnumValue.
      */
    EnumMapping(
        const eC_String& rkTypeTag,
        eC_Int iEnumValue,
        const eC_String& rkValueName) :
        m_kTypeTag(rkTypeTag),
        m_iEnumValue(iEnumValue),
        m_kValueName(rkValueName)
    {}

    EnumMapping() :
        m_kTypeTag(""),
        m_iEnumValue(0),
        m_kValueName("")
    {}

    /// The XML tag used when reading from or writing the enumeration value to a stream.
    eC_String m_kTypeTag;

    /// The enumeration value.
    eC_Int m_iEnumValue;

    /// The enumeration value as a string.
    eC_String m_kValueName;
};

/// Describes a custom attribute repetition.
struct AttributeRepetitionDescriptor
{
    /** Constructs a repetition descriptor.
        @param rkRepetitionTitle The title of the repetition.
        @param rkXMLTag The XML tag of the repetition.
        @param rkAttributeValue The attribute value of the repetition as string.
        @param kAttributeType The attribute type of the repetition as enum.
      */
    AttributeRepetitionDescriptor(
        const eC_String &rkRepetitionTitle,
        const eC_String &rkXMLTag,
        const eC_String &rkAttributeValue,
        CGUICommonEnums::AttributeType kAttributeType) :
        m_kRepetitionTitle(rkRepetitionTitle),
        m_kXMLTag(rkXMLTag),
        m_kAttributeValue(rkAttributeValue),
        m_kAttributeType(kAttributeType)
    {}

    AttributeRepetitionDescriptor() :
        m_kRepetitionTitle(""),
        m_kXMLTag(""),
        m_kAttributeValue(""),
        m_kAttributeType(CGUICommonEnums::AT_COMMENT)
    {}

    /// The repetition title as a string (The attribute name of the tag containing the number of repetitions).
    /// Marks the end of a repetition lists (XMLTag and AttributeValue have to be empty strings).
    eC_String m_kRepetitionTitle;

    /// The repetition XML tag as a string.
    eC_String m_kXMLTag;

    /// The repetition attribute value as a string.
    eC_String m_kAttributeValue;

    /// The repetition attribute type enum.
    CGUICommonEnums::AttributeType m_kAttributeType;
};

typedef std::vector<EnumMapping> EnumMappingVector;
typedef std::vector<AttributeRepetitionDescriptor> RepetitionVector;

#endif
