/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __FNTWRAPFREETYPEGL__H__
#define __FNTWRAPFREETYPEGL__H__

#include "FntWrapFreeType.h"
#include "FntWrapGLFontAtlas.h"

/// Base class for FreeType-based font wrapper using GL textures.
/** This class uses the class FntWrapFreeTypeGLFontAtlas for displaying FreeType
    fonts using the OpenGL-Wrapper.
  */
class CFntWrapFreeTypeGL : public CFntWrapFreeType
{
public:
    /** Creates the instance of this wrapper.
        @param uiCacheSize Maximal number of simultaneously cached characters. If parameter
                           equals zero the cache will automatically be resized to be
                           large enough to contain all required characters.
        @param uiMaxNumFaces maximum number of faces freetype will try to store
        @param uiMaxNumSizeObjects maximum number of size object freetype will try to store
        @return True if successful, False otherwise
    */
    static eC_Bool CreateInstance(
        const eC_UInt &uiCacheSize = 0,
        const eC_UInt& uiMaxNumFaces = 10,
        const eC_UInt& uiMaxNumSizeObjects = 10);

protected:
    /** Constructor.
        @param uiCacheSize Maximal number of simultaneously cached characters. If parameter
                           equals zero the cache will automatically be resized to be
                           large enough to contain all required characters.
        @param uiMaxNumFaces maximum number of faces freetype will try to store
        @param uiMaxNumSizeObjects maximum number of size object freetype will try to store
    */
    CFntWrapFreeTypeGL(
        const eC_UInt& uiCacheSize,
        const eC_UInt& uiMaxNumFaces,
        const eC_UInt& uiMaxNumSizeObjects);

    /// Destructor.
    virtual ~CFntWrapFreeTypeGL();

    virtual void LoadFont (const FontResource_t &eFontID);
    virtual void UnloadFont (const FontResource_t &eFontID);

    virtual void GetGlyphMetrics(const eC_Char cChar, GlyphMetrics_t& kMetrics);

    virtual eC_Int GetAscender() const;
    virtual eC_Int GetDescender() const;
    virtual eC_Int GetInternalLeading() const;

    virtual eC_Bool SetNOFFonts(const eC_UInt uiNOFFonts);

    /** Font-atlas used for grouping the glyphs in a more efficient way
    for faster access during rendering.
    */
    eC_TArray<CFntWrapGLFontAtlas*> m_apGLFontAtlas;

private:
    /** Renders a text to screen or calculates the required space.
        Because of redundant code of Text() and RequiredSpace() this function implements both.
        Text() and RequiredSpace() are using this function.
        @param[in] lpString The string which will be rendered or which dimensions will be calculated.
        @param[in] eTextOperation Use RENDER_TEXT to render the text or GET_REQUIRED_SPACE to calculate the required space.
        @param[in] vX1 The position of the text (x) (only used for text rendering).
        @param[in] vY1 The position of the text (y) (only used for text rendering).
        @param[out] vWidth The width of the required space.
        @param[out] vHeight The height of the required space.
        @param[in]  vWidthMax The maximum width possible the text can have.
        @param[out] uiNumFittingChars The number of characters that fit into the given maximum width.
    */
    void RenderTextInternal(
        const eC_String *lpString,
        const TextOperation_t &eOperation,
        const eC_Value& vX1, const eC_Value& vY1,
        eC_Value& vWidth, eC_Value& vHeight,
        eC_Value vWidthMax, eC_UInt &uiNumFittingChars);

    virtual void RenderGlyphToScreen(const eC_Value& vX1, const eC_Value& vY1, CGUIImageData& kGlyphImage);

    /** Copy-constructor.
        Dummy implementation to avoid unintended use of compiler-generated default
    */
    CFntWrapFreeTypeGL(const CFntWrapFreeTypeGL& kSource);

    /** Operator= method.
        Dummy implementation to avoid unintended use of compiler-generated default
    */
    CFntWrapFreeTypeGL& operator=(const CFntWrapFreeTypeGL& kSource);

    void AddGlyph(
        eC_TArray<eC_Value>& avVertices,
        const eC_UInt& uiArrayIndex,
        const eC_Value& vX,
        const eC_Value& vY,
        const eC_Int& iWidth,
        const eC_Int& iHeight,
        const eC_Value& vULeft,
        const eC_Value& vURight,
        const eC_Value& vVTop,
        const eC_Value& vVBottom);
};
#endif
