/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __FONTCACHE_H__
#define __FONTCACHE_H__

#include "eC_Types.h"
#include "eC_TArray.h"

#include "GUIImageData.h"
#include "GUIFontResource.h"

/// FontCache is the base-class for glyph-cache.
/** It stores the cache information for
one Font.
*/
class CFontCache
{
public:
    /// Struct representing one character inside the cache array
    /// the size of this struct has direct impact on the duration of the sorting!!
    struct CachedCharInfo_t
    {
        CachedCharInfo_t() :
            m_iBitmapGlyphLeft(0),
            m_iBitmapGlyphTop(0),
            m_iBitmapRows(0),
            m_iBitmapWidth(0),
            m_uiWidth(0),
            m_uiTimestamp(0),
            m_uiXAdvance(0),
            m_pkImageData(NULL)
        {}

        eC_Int  m_iBitmapGlyphLeft; ///< Glyph bitmap position offset
        eC_Int  m_iBitmapGlyphTop; ///< Glyph bitmap position offset
        eC_Int  m_iBitmapRows; ///< Bitmap height
        eC_Int  m_iBitmapWidth; ///< Width of the bitmap itself
        eC_UInt m_uiWidth; ///< Total width of character with space on the left and right
        eC_UInt m_uiTimestamp; ///< Stores timestamp of last access
        eC_UInt m_uiXAdvance; ///< x-advance
        CGUIImageData* m_pkImageData; ///< image-data
    };

    /// struct for holding the unicode and a pointer to the related info-block
    struct CachedChar_t
    {
        CachedChar_t() :
            m_uiUnicode(0),
            m_pkInfo(NULL)
        {}

        eC_UInt m_uiUnicode; ///< unicode
        CachedCharInfo_t* m_pkInfo; ///< info-block
    };

public:
    /** Constructor
    @param eFontID ID of selected font.
    @param uiInitialCacheSize initial number of simultaneously cached characters. If parameter
    equals zero the cache will automatically be resized to be
    large enough to contain all required characters.
    @param bAutoResize if true the cache gets resized if not big enough to hold new entries
    @param uiResizeBy number of elements which are added on resize
    */
    CFontCache(
        const FontResource_t& eFontID,
        const eC_UInt& uiInitialCacheSize,
        const eC_Bool& bAutoResize,
        const eC_UInt& uiResizeBy);

    /** Destructor */
    virtual ~CFontCache();

    /** This method returns a struct with the wanted character's information.
    It adds a character to the cache array if it does not exist.
    @param uiUnicode Unicode of wanted character.
    @return Struct of wanted character.
    */
    CachedChar_t GetChar(const eC_UInt &uiUnicode);

    /** Returns the height of current font.
    @return Font's height.
    */
    inline eC_Int GetHeight() const { return m_iHeight; }

    /** Returns the ascender of the current font.
    @return Font's ascender.
    */
    inline eC_Int GetAscender() const { return m_iAscender; }

    /** Returns the descender of the current font.
    @return Font's descender.
    */
    inline eC_Int GetDescender() const { return m_iDescender; }

    /** Returns the internal leading of the current font.
    @return Font's internal leading.
    */
    inline eC_Int GetInternalLeading() const { return m_iInternalLeading; }

    /** get internal face from font
    @return face
    */
    void* GetFace() { return m_pkFace; }

protected:
    /** Adds a character to the cache array and sorts array afterwards.
    @param uiUnicode Unicode to add to array.
    @param uiPos Position in array where the character should be added.
    @return Struct of added character.
    */
    virtual CachedChar_t AddChar(const eC_UInt &uiUnicode, const eC_UInt &uiPos) = 0;

    /** Removes a character from array.
    @param uiPos Position of the character to remove.
    */
    virtual void RemoveChar(const eC_UInt &uiPos);

    /** Searches for a character inside the cache array by using divide and
    conquer.
    @param uiUnicode Character to search for.
    @return Position in array of -1 if not found.
    */
    eC_Int FindChar(const eC_UInt &uiUnicode);

    /** Swaps two given values.
    @param pPosA Pointer of the first value.
    @param pPosB Pointer of the second value.
    */
    void Swap(CachedChar_t* pPosA, CachedChar_t* pPosB);

    /** Sorts the cache list by unicode in ascending order using bubble sort
    algorithm.*/
    void Sort();

    /** Searches the list for the lowest timestamp and return that position.
    @return Lowest timestamp position in array.
    */
    eC_UInt FindLowestTimestampPos();

    /** Copy-constructor.
    Dummy implementation to avoid unintended use of compiler-generated default
    @param kSource
    */
    CFontCache(const CFontCache& kSource);

    /** Operator= method.
    Dummy implementation to avoid unintended use of compiler-generated default
    @param kSource
    @return
    */
    CFontCache& operator=(const CFontCache& kSource);

protected:
    FontResource_t m_eFontID; ///< loaded font
    eC_Int m_iAscender; ///< ascender
    eC_Int m_iDescender; ///< descender
    eC_Int m_iInternalLeading; ///< internal leading
    eC_Int m_iHeight; ///< height of font
    eC_UInt m_uiNextGlyphIndex; ///< index of next glyph
    eC_Bool m_bUseTimeStamp; ///< timestamp
    eC_UInt m_uiTimer; ///< current timer

    eC_Bool m_bAutoResize; ///< auto-resize
    eC_UInt m_uiResizeBy; ///< resize by amount

    eC_TArray<CachedChar_t> m_asCache; ///< The dynamic cache array.

    eC_Bool m_bArrayNeedsSorting; ///< sorting needed after modification
    void* m_pkFace; ///< face of font
};

#endif
