/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef GUIINPUTMEDIA__H_
#define GUIINPUTMEDIA__H_

#include "eC_Semaphore.h"
#include "eC_Types.h"
#include "eC_TList_doubleLinked.h"

#include "GUIComponentManager.h"

/// Retrieve pointer to inputmedia instance
#define GETINPUTMEDIA CGUIComponentManager::GetInstance().GetInputMedia()

class CGUI;
class CGUIObject;
class CGUIEvent;

/** Base-class for all input devices that are attachable to the GUI.
    @see @ref sec_porting_input "Porting User Input for Guiliani"
*/
class CGUIInputMedia
{
    friend class CGUIComponentManager; ///< @todo for private access

public:
    /// rotation of input-device
    enum Rotation_t
    {
        ROTATION_0,
        ROTATION_90,
        ROTATION_180,
        ROTATION_270
    };

    /// how events are collected
    enum AcquisitionType_t
    {
        Polling,
        Callback
    };

public:
    /** Delete Instance
    */
    static void DeleteInstance();

    /** Retrieves a platform-specific event and creates an appropriate Guiliani event.
        The supplied parameter specifies the time in milliseconds until the next processing of
        animation timers or commands is required. An input media may use this to avoid unnecessary
        event polling. It MUST return from GetEvent after the specified time, though.
        @param uiIdleTime specifies the maximum idle time in ms.
        @return Pointer to a newly generated Guiliani event, or NULL if nothing happened.
    */
    CGUIEvent* GetEvent(eC_UInt uiIdleTime);

    /** add a new event into the queue (if used)
    @param pkEvent event
    */
    void AddEvent(CGUIEvent* pkEvent);

    /** this method will be called if the input device uses polling to receive events
    */
    virtual void PollEvent() {}

    /** Forces the input media to quit idling immediately.
        Called by the framework if the input media is requested to stop idling immediately.
        Classes derived from GUIInputMedia should implement StopIdle in such a way, that it interrupts any
        idling/sleeping code in GetEvent().
        virtual is needed if sub-classes need to do extra stuff
    */
    virtual void StopIdle();

    /** Set the rotation of the input-device
    @param eRotation
    */
    void SetRotation(const Rotation_t& eRotation);

    /** Get the rotation of the input-device
    @return rotation
    */
    Rotation_t GetRotation() const;

    /** Activates debug-mode for input-device
        Will output messages for every input and can be used for calibrating input-device
        @param bDebugMode true for active debug-mode
    */
    void SetDebugMode(const eC_Bool& bDebugMode);
    
    /** Return if debug-mode for input-device is active
        @return true if active
    */
    eC_Bool GetDebugMode() const;

    /** set the size for the input-device
    if negative values are used the axis is inverted
    @param iSizeX width
    @param iSizeY height
    */
    virtual void SetSize(const eC_Int& iSizeX, const eC_Int& iSizeY);

    /** get the size of the input-device
    @param riSizeX width
    @param riSizeY height
    */
    virtual void GetSize(eC_Int& riSizeX, eC_Int& riSizeY) const;

    /** set the offset for the input-device
    @param iOffsetX x-offset
    @param iOffsetY y-offset
    */
    virtual void SetOffset(const eC_Int& iOffsetX, const eC_Int& iOffsetY);

    /** get the offset of the input-device
    @param riOffsetX x-offset
    @param riOffsetY y-offset
    */
    virtual void GetOffset(eC_Int& riOffsetX, eC_Int& riOffsetY) const;

    /** set multiple touches supported
    @param bMultipleTouches
    */
    virtual void SetMultipleTouches(const eC_Bool& bMultipleTouches);

    /** get if multiple touches supported
    @return true if multiple touches supported
    */
    virtual eC_Bool GetMulitpleTouches() const;

    /** set the interval used for internal sleep
    @param uiInterval
    */
    void SetSleepInterval(const eC_UInt& uiInterval);

    /** get the current sleep-interval
    @return interval
    */
    eC_UInt GetSleepInterval() const;

    /** set the maximum number of events in the list before events are dropped
    @param uiMaxEventThreshold
    */
    void SetMaxEventThreshold(const eC_UInt& uiMaxEventThreshold);

    /** get the maximum number of events
    @return maximum number of events
    */
    eC_UInt GetMaxEventThreshold() const;

    /** set the minimum number of events in the list before event dropping is disabled
    @param uiMinEventThreshold
    */
    void SetMinEventThreshold(const eC_UInt& uiMinEventThreshold);

    /** get the minimum number of events
    @return minimum number of events
    */
    eC_UInt GetMinEventThreshold() const;

    /** retrieve and remove an event from the queue (if used)
    @return event
    */
    CGUIEvent* RetrieveEvent();

protected:
    /** constructor
    @param bQueuedEvents if true an internal list will be used to save events,
           otherwise only the last acquired event will be saved
    @param eType acquisition type can be POLLING or CALLBACK
    */
    CGUIInputMedia(const eC_Bool& bQueuedEvents = false, const AcquisitionType_t& eType = Polling);

    /** destructor
    */
    virtual ~CGUIInputMedia();

protected:
    eC_UInt m_uiIdleTime; ///< maximum time to spend
    eC_Bool m_bStopIdle; ///< stop idle when waiting for events

    Rotation_t m_eRotation; ///< current rotation used for input-device

    eC_Bool m_bDebugMode; ///< debug-mode for input-device

    eC_Int m_iSizeX; ///< width
    eC_Int m_iSizeY; ///< height

    eC_Int m_iOffsetX; ///< x-offset
    eC_Int m_iOffsetY; ///< y-offset

    eC_UInt m_uiSleepInterval; ///< sleep interval

    eC_Bool m_bMultipleTouches; ///< supports multiple touches

private:
    /** Copy-constructor.
        Dummy implementation to avoid unintended use of compiler-generated default    */
    CGUIInputMedia(const CGUIInputMedia& kSource);

    /** Operator= method.
        Dummy implementation to avoid unintended use of compiler-generated default    */
    CGUIInputMedia& operator=(const CGUIInputMedia& kSource);

private:
    eC_Bool m_bQueuedEvents;
    eC_Semaphore m_kEventListSemaphore;
    eC_TListDoubleLinked<CGUIEvent*> m_kEventList;
    eC_UInt m_uiMaxEventThreshold;
    eC_UInt m_uiMinEventThreshold;

    CGUIEvent* m_pkLastEvent;

    eC_Bool m_bIgnoreEvents;

    AcquisitionType_t m_eType;
};

#endif
