/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GFXENV_H__
#define __GFXENV_H__

#include "eC_Types.h"
#include "eC_String.h"

#include "GUICommonEnums.h"
#include "eC_TList_doubleLinked.h"

#include "GUIRect.h"

#include "GUILayerConfig.h"

#include "GUIComponentManager.h"

#define GETGFXENV CGUIComponentManager::GetInstance().GetGfxEnv()

class CGfxWrap;

/** This is the base-class for all GfxEnvXXX-classes dealing with environment used by the GfxWrap

This class is used to set up the environment for the actual GfxWrap with a generic interface.
It will provide the size, buffers and all necessary data. Additionally it will hold all hardware layers.

Typical order of events:

After the frame has been drawn by the GfxWrap into the currently active render-buffer the refresh is done
Inside the refresh the following things are done for each layer
- UpdateLayerBuffers to switch to the next buffer

After that EndOfRefresh and RefreshScreen with each invalidated region is called

Depending on the actual underlying API UpdateLayerBuffers can be used to execute cache-invalidation
on the buffers currently used for drawing. Additionally the switch of buffers for the display-hardware
can be done.

In EndOfRefresh the switch of all used buffers should already be done, thus it can be used to wait for a
vertical sync-signal.

RefreshScreen can be used to copy the regions that have been redrawn in the last frame over to the new
drawing-buffer to start the next frame.
*/
class CGfxEnv
{
    // for calling destructor
    friend class CGUIComponentManager;

public:
    /** Surface-mode */
    enum SurfaceMode_t
    {
        SM_OPENGL, ///< use OpenGL for surface
        SM_FRAMEBUFFER, ///< use framebuffer for surface
        SM_VULKAN, /// use Vulkan for surface
    };

public:
    /** Create Instance
    @param eSurfaceMode surface-mode
    @param eScreenFormat screen-format
    @param uiWidth width
    @param uiHeight height
    @param uiBufferCount number of buffers to use
    @param ppkBuffers pointers to buffers
    @param bResizable true if resizable
    @return true if successful
    */
    static eC_Bool CreateInstance(
        const SurfaceMode_t& eSurfaceMode,
        const CGUICommonEnums::ImageType_t& eScreenFormat,
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight,
        const eC_UInt& uiBufferCount = 1,
        void** ppkBuffers = NULL,
        const eC_Bool& bResizable = false);

    /// called to destroy and detach GfxEnv
    static void DeleteInstance();

    /** Set the screen position if available
    @param uiX
    @param uiY
    */
    virtual void SetScreenPosition(const eC_UInt& uiX, const eC_UInt& uiY);

    /** Set the screen-size
    this will normally result in recreating the surfaces
    @param uiWidth
    @param uiHeight
    */
    virtual void SetScreenSize(const eC_UInt& uiWidth, const eC_UInt& uiHeight);

    /** Set if VSync is enabled
    @param bVSyncEnabled set to true if VSync is enabled
    */
    virtual void SetVSyncEnabled(const eC_Bool& bVSyncEnabled);

    /** Is surface created
    @return true if created
    */
    virtual eC_Bool IsCreated() const;

    /** Retrieve the Surface-Mode
    @return surface-mode
    */
    virtual SurfaceMode_t GetSurfaceMode() const;

    /** Retrieve the Screen-Format (e.g. RGB565)
    @return screen-format
    */
    virtual CGUICommonEnums::ImageType_t GetColorFormat() const;

    /** Get the x-position of the surface
    @return x-position
    */
    virtual eC_UInt GetXPos() const;

    /** Get the y-position of the surface
    @return y-position
    */
    virtual eC_UInt GetYPos() const;

    /** Get the width of the surface
    @return width
    */
    virtual eC_UInt GetWidth() const;

    /** Get the height of the surface
    @return height
    */
    virtual eC_UInt GetHeight() const;

    /** Get the BPP of the surface
    @return BPP
    */
    virtual eC_UByte GetBPP() const;

    /** Get Resizable
    @return resizable
    */
    virtual eC_Bool IsResizable() const;

    /** Get if merge is needed
    @return merge needed
    */
    virtual eC_Bool NeedsManualMerge() const;

    /** Get if copy is supported
    @return copy supported
    */
    virtual eC_Bool SupportsCopy() const;

    /** Get number of used buffers
    @return buffer-count
    */
    virtual eC_UInt GetBufferCount() const;

    /** Return the name of the environment
    @return name of environment
    */
    virtual eC_String GetName() const;

    /** return the used framebuffer if any
    @return framebuffer or NULL
    */
    virtual void** GetFrameBuffers() const { return m_ppkBuffers; }

    /** Return if VSync is enabled
    @return true if VSync is enabled
    */
    virtual eC_Bool IsVSyncEnabled() const { return m_bVSyncEnabled; }

    /** Refresh the screen within the given invalidate area.
        This can be used to copy regions of the display-buffer to the drawing-buffer
        @param rkInvalidatedArea This area will be refreshed.
        @return True if OK
     */
    virtual eC_Bool RefreshScreen(const CGUIRect& rkInvalidatedArea) { return false; }

    /** Refresh multiple parts of the screen
        @param kInvalidatedRectList invalidated areas
        @return True if OK
    */
    virtual eC_Bool RefreshScreen(const eC_TListDoubleLinked<CGUIRect>& kInvalidatedRectList);

    /** Set the clip rectangle.
        @param rkNewClipRect The new clipping rectangle.
     */
    virtual void SetCliprect(const CGUIRect& rkNewClipRect) {}

    /** Set the window caption
    @param kCaption
    */
    virtual void SetWindowCaption(const eC_String& kCaption) {}

    /** Swap buffers
    @return true if successful
    */
    virtual eC_Bool SwapBuffers() { return false; }

    /**
    Initialize display
    @return 1 on success, else 0
    */
    virtual eC_UInt InitDisplay() { return 1; }

    /**
    De-Initialize display
    */
    virtual void DeInitDisplay() {}

    /** ------ layer functionality ------ */

    /**
    Get the internal index for given layer-id
    @param uiLayerID id of requested layer
    @param ruiLayerIndex internal index
    @return true if layer exists, otherwise false
    */
    eC_Bool GetLayerIndexForID(const eC_UInt& uiLayerID, eC_UInt& ruiLayerIndex) const;

    /**
    Set the given layer-id as the active
    all following access for rendering will be sent to the layer
    @param uiLayerID id of layer
    @param bMarkAsModified layer will be marked as modified upon switch (default: false)
    @return true if layer exists, otherwise false
    */
    eC_Bool SetActiveLayerID(const eC_UInt& uiLayerID, const eC_Bool& bMarkAsModified = false);

    /**
    Return the id of the currently active layer
    @return id of active layer
    */
    eC_UInt GetActiveLayerID() const;

    /**
    Return if layer was recently modified
    @param uiLayerID id of layer
    @return True if modified
    */
    eC_Bool IsLayerModified(const eC_UInt& uiLayerID) const;

    /**
    Update buffers for layer
    @param uiLayerID id of layer
    @param pkRenderBuffer currently used buffer for rendering
    @param pkDisplayBuffer currently used buffer for display
    @param uiWidth width of layer
    @param uiHeight height of layer
    @return true if successful
    */
    virtual eC_Bool UpdateLayerBuffers(const eC_UInt& uiLayerID, void* pkRenderBuffer, void* pkDisplayBuffer, const eC_UInt& uiWidth, const eC_UInt& uiHeight);

    /** Set information for the main-layer
    @param uiLayerID the id for the layer (default: 0)
    @param bCached true if the layer-memory is cached (default: false)
    */
    void AddMainLayer(const eC_UInt& uiLayerID = 0, const eC_Bool& bCached = false);

    /** Set the configuration of the main-layer
    @param uiLayerID set layer-id (default: 0)
    @param bCached set to true if layer-memory is cached (default: false)
    */
    void SetMainLayerProperties(const eC_UInt& uiLayerID, const eC_Bool& bCached);

    /**
    Add layer to internal configuration. should only be called during startup of application
    @param kLayerConfig layer-configuration
    @return true if successful
    */
    eC_Bool AddLayer(const LayerConfiguration_t& kLayerConfig);

    /**
    Initialize layers after all layers have been added
    */
    void InitLayers();

    /**
    Return number of configured layers
    @return number of layers
    */
    eC_UInt GetNumberOfLayers() const;

    /**
    Get configuration for given layer. Note: this will need an index instead of id
    @param uiLayerIndex index of layer in internal data
    @param rkLayerConfig configuation of layer
    @return true if config was found
    */
    eC_Bool GetLayerConfig(const eC_UInt& uiLayerIndex, LayerConfiguration_t& rkLayerConfig) const;

    /**
    Get configuration for layer with given ID
    @param uiLayerID id of layer
    @param rkLayerConfig configuation of layer
    @return true if config was found
    */
    eC_Bool GetLayerConfigForID(const eC_UInt& uiLayerID, LayerConfiguration_t& rkLayerConfig) const;

    /**
    Get configuration for main-layer
    @param rkLayerConfig configuation of layer
    @return true if config was found
    */
    eC_Bool GetLayerConfigForMainLayer(LayerConfiguration_t& rkLayerConfig) const;

    /**
    Set the layer to visible
    @param uiLayerID id of layer
    @param bVisible visibility of layer
    @return true if layer was found
    */
    eC_Bool SetLayerVisible(const eC_UInt& uiLayerID, const eC_Bool& bVisible);

    /**
    Get current visibility of layer
    @param uiLayerID id of layer
    @return true if visible, else false
    */
    eC_Bool IsLayerVisible(const eC_UInt& uiLayerID) const;

    /**
    Set the position for the given layer
    @param uiLayerID id of layer
    @param iXPos x-position
    @param iYPos y-position
    @return true if layer was found
    */
    eC_Bool SetLayerPosition(const eC_UInt& uiLayerID, const eC_Int& iXPos, const eC_Int& iYPos);

    /**
    Get the current position for the given layer
    @param uiLayerID id of layer
    @param iXPos x-position
    @param iYPos y-position
    @return true if layer was found
    */
    eC_Bool GetLayerPosition(const eC_UInt& uiLayerID, eC_Int& iXPos, eC_Int& iYPos);

    /**
    Set the visible size for the given layer
    @param uiLayerID id of layer
    @param uiWidth visible width
    @param uiHeight visible height
    @return true if layer was found
    */
    eC_Bool SetLayerVisibleSize(const eC_UInt& uiLayerID, const eC_UInt& uiWidth, const eC_UInt& uiHeight);

    /**
    Get the visible size for the given layer
    @param uiLayerID id of layer
    @param uiWidth visible width
    @param uiHeight visible height
    @return true if layer was found
    */
    eC_Bool GetLayerVisibleSize(const eC_UInt& uiLayerID, eC_UInt& uiWidth, eC_UInt& uiHeight);

    /**
    Set the visible position for the given layer
    @param uiLayerID id of layer
    @param iXPos x-position
    @param iYPos y-position
    @return true if layer was found
    */
    eC_Bool SetLayerVisiblePos(const eC_UInt& uiLayerID, const eC_Int& iXPos, const eC_Int& iYPos);

    /**
    Set the alpha-value for blending between layer for the given layer.
    @param uiLayerID id of layer
    @param ubAlpha alpha-value
    @return true if layer was found
    */
    eC_Bool SetLayerAlpha(const eC_UInt& uiLayerID, const eC_UByte& ubAlpha);

    /**
    Get the current alpha-value for the given layer.
    @param uiLayerID id of layer
    @return alpha-value
    */
    eC_UByte GetLayerAlpha(const eC_UInt& uiLayerID) const;

    /**
    Set the chroma-key for blending between layer for the given layer.
    @param uiLayerID id of layer
    @param bChromaKeyActive true if chroma-keying is active
    @param uiChromaKey the color for chroma-keying
    @param ubAlpha alpha-value for replacement
    @return true if layer was found
    */
    eC_Bool SetLayerChromaKey(const eC_UInt& uiLayerID, const eC_Bool& bChromaKeyActive, const eC_UInt& uiChromaKey, const eC_UByte& ubAlpha);

    /**
    Get the current chroma-key settings for the given layer.
    @param uiLayerID id of layer
    @param bChromaKeyActive true if chroma-keying is active
    @param uiChromaKey the color for chroma-keying
    @param ubAlpha alpha-value for replacement
    @return true if layer was found
    */
    eC_Bool GetLayerChromaKey(const eC_UInt& uiLayerID, eC_Bool& bChromaKeyActive, eC_UInt& uiChromaKey, eC_UByte& ubAlpha) const;

    /**
    Get the size of the buffer attached to the given layer.
    @param uiLayerID id of layer
    @param uiWidth width of layer
    @param uiHeight height of layer
    @param ubBPP bytes per pixel
    @return true if layer was found
    */
    eC_Bool GetLayerBufferSize(const eC_UInt& uiLayerID, eC_UInt& uiWidth, eC_UInt& uiHeight, eC_UByte& ubBPP);

    /**
    Update layer according to reason.
    This will be called when attributes of the layer (position, size, etc) have changed
    @param kLayerConfig configuration of updated layer
    @param eUpdateReason reason of update
    */
    virtual void UpdateLayer(const LayerConfiguration_t& kLayerConfig, const LayerUpdate_t& eUpdateReason) {}

    /**
    Get the currently active buffer which is used for rendering
    @param uiLayerID id of layer
    @return address of buffer or NULL if layer was not found
    */
    void* GetRenderBuffer(const eC_UInt& uiLayerID) const;

    /**
    Get the currently active buffer which is used for display
    @param uiLayerID id of layer
    @return address of buffer or NULL if layer was not found
    */
    void* GetDisplayBuffer(const eC_UInt& uiLayerID) const;

    /**
    Refresh layer after content has been changed. This applies merely to layers containing user-defined content
    @param uiLayerID id of layer to refresh
    */
    void InvalidateLayer(const eC_UInt& uiLayerID);

    /**
    This will update the positions for all layers, BEFORE the redraw is done.
    */
    void UpdateLayerPositions();

    /**
    This will link an object to a specific layer. this info is used in UpdateLayerPositions
    @param uiLayerID id of layer
    @param pkObject pointer to object
    */
    void SetObjectForLayerID(const eC_UInt& uiLayerID, CGUIObject* pkObject);

    /**
    Set the global alpha-value, which is used to correctly blend
    the layers during a transition
    @param ubGlobalAlpha
    */
    void SetGlobalAlpha(const eC_UByte& ubGlobalAlpha);

    /**
    Return the currently set global alpha-value
    @return global alpha
    */
    eC_UByte GetGlobalAlpha() const;

    /**
    Calculate the alpha-value according to the global alpha
    @param ubAlpha input alpha-value
    @return applied alpha
    */
    eC_UByte ApplyGlobalAlpha(const eC_UByte& ubAlpha) const;

    /**
    This will be called at the start of refresh
    */
    virtual void StartOfRefresh() {}

    /**
    This will be called when all layers have been refreshed.
    To wait for V-sync this would be the right place.
    */
    virtual void EndOfRefresh() {}

    /**
    This will set the id of the main-layer
    @param uiLayerID id of layer
    */
    void SetMainLayerID(const eC_UInt& uiLayerID);

    /**
    Return the ID of the main-layer
    @return main-layer ID
    */
    eC_UInt GetMainLayerID() const;

    /**
    Return if layer is set to be used as main-layer
    @param uiLayerID id of layer
    @return True if main-layer
    */
    eC_Bool IsMainLayer(const eC_UInt& uiLayerID) const;

    /** Check if Wrapper is NULL-implementation
    @return true if NULL
    */
    virtual eC_Bool IsNull() { return false; }

    /** Set the GfxWrapper
    @param pkGfxWrap
    */
    void SetGfxWrap(CGfxWrap* pkGfxWrap) { m_pkGfxWrap = pkGfxWrap; }

    /** Return the number of bytes per pixel for pixel-format
    @param ePixFormat
    @return bpp
    */
    eC_UByte GetBytesPerPixel(const CGUICommonEnums::ImageType_t& ePixFormat) const;

    /** Set a device
    @param pkDevice
    */
    virtual void SetDevice(void* pkDevice) {}

    /** Return merge-buffer
    @return NULL by default
    */
    virtual void* GetMergeBuffer() const { return NULL; }

    /** return display-handle if any
    @return NULL by default
    */
    virtual void* GetDisplayHandle() const { return NULL; }

    /** return window-handle if any
    @return NULL by default
    */
    virtual void* GetWindowHandle() const { return NULL; }

protected:
    /** Constructor
    @param eSurfaceMode OpenGL or Framebuffer
    @param eScreenFormat screen-format
    @param uiWidth width
    @param uiHeight height
    @param uiBufferCount number of buffers to use
    @param ppkBuffers buffers to use
    @param bResizable true if resizable
    */
    CGfxEnv(
        const SurfaceMode_t& eSurfaceMode,
        const CGUICommonEnums::ImageType_t& eScreenFormat,
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight,
        const eC_UInt& uiBufferCount = 1,
        void** ppkBuffers = NULL,
        const eC_Bool& bResizable = false);

    /** Destructor
    */
    virtual ~CGfxEnv();

    /** Get the number of bits per pixel from the screen-format
    @return number of bits
    */
    eC_UByte GetBPPFromScreenFormat() const;

    /**
    Initialize layer
    this can be used to execute platform-specific code
    is called when layer gets added to DCWrap

    @param kLayerConfig layer-configuration
    */
    virtual void InitLayer(const LayerConfiguration_t& kLayerConfig) {}

    /**
    Update settings of main-layer
    @param eColorFormat color-format
    @param uiWidth width
    @param uiHeight height
    @param uiBufferCount number of buffers
    @param ppkBuffers pointer to buffers
    */
    void UpdateMainLayer(
        const CGUICommonEnums::ImageType_t& eColorFormat,
        const eC_UInt& uiWidth, const eC_UInt& uiHeight,
        const eC_UInt& uiBufferCount,
        void** ppkBuffers);

protected:
    SurfaceMode_t m_eSurfaceMode; ///< surface mode
    CGUICommonEnums::ImageType_t m_eColorFormat; ///< color-format
    eC_UInt m_uiWidth; ///< width of the surface
    eC_UInt m_uiHeight; ///< height of the surface
    eC_UByte m_ubBPP; ///< BPP of the surface
    eC_UInt m_uiBufferCount; ///< number of buffers used
    void** m_ppkBuffers; ///< buffers for the main-screen
    eC_Bool m_bResizable; ///< resizable
    eC_Bool m_bNeedsManualMerge; ///< if supports merge
    eC_Bool m_bSupportsCopy; ///< if supports copy
    eC_String m_kName; ///< name

protected:
    eC_TArray<LayerConfiguration_t> m_akLayers; ///< : *DCLayer_t, one DCLayer_t struct per layer storing general settings
    eC_UInt m_uiActiveLayerID; ///< : unsigned int = Active Layer
    eC_UInt m_uiActiveLayerIndex; ///< index of currentl< active layer

    CGfxWrap* m_pkGfxWrap; ///< pointer to GfxWrapper

    eC_Int m_iMainLayerID; ///< id of main-layer
    eC_UByte m_ubGlobalAlpha; ///< global alpha for blending
    eC_Bool m_bVSyncEnabled; ///< if vsync is enabled
};
#endif
