/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GFX_ENV_WIN__H__
#define __GFX_ENV_WIN__H__

#include "GfxEnv.h"

#include "GUIRect.h"
#include <windows.h>

/// Implements windows GDI specific features.

/**
    This class implements the Windows GDI specific features, which may be
    used in different graphic wrappers within the Windows environment.
*/
class CGfxEnvWin : public CGfxEnv
{
public:
    /** Create instance
    @param hInstance The instance of the window.
    @param eSurfaceMode surface-mode
    @param eScreenFormat screen-format
    @param uiWidth width
    @param uiHeight height
    @param uiBufferCount number of buffers
    @param bResizable true if resizable
    @return true if successful
    */
    static eC_Bool CreateInstance(
        const HINSTANCE& hInstance,
        const SurfaceMode_t& eSurfaceMode,
        const CGUICommonEnums::ImageType_t& eScreenFormat,
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight,
        const eC_UInt& uiBufferCount = 1,
        const eC_Bool& bResizable = false);

    /** Get handle of window
    @return window-handle
    */
    virtual void* GetWindowHandle() const
    {
        return m_hWnd;
    }

protected:
    /** Constructor.
    @param hInstance The instance of the window.
    @param eSurfaceMode surface-mode
    @param eScreenFormat
    @param uiWidth width
    @param uiHeight height
    @param uiBufferCount number of buffers to use
    @param bResizable true if resizable
    */
    CGfxEnvWin(
        const HINSTANCE& hInstance,
        const SurfaceMode_t& eSurfaceMode,
        const CGUICommonEnums::ImageType_t& eScreenFormat,
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight,
        const eC_UInt& uiBufferCount = 1,
        const eC_Bool& bResizable = false);

    /** Destructor
    */
    virtual ~CGfxEnvWin();

protected:
    virtual void SetScreenPosition(const eC_UInt& uiX, const eC_UInt& uiY);
    virtual void SetScreenSize(const eC_UInt& uiWidth, const eC_UInt& uiHeight);

    virtual eC_UInt GetXPos() const;
    virtual eC_UInt GetYPos() const;
    virtual eC_UInt GetWidth() const;
    virtual eC_UInt GetHeight() const;

    virtual eC_Bool RefreshScreen(const CGUIRect& rkInvalidatedArea);

    virtual void** GetFrameBuffers() const;

    virtual void SetWindowCaption(const eC_String& kCaption);

    virtual void InitLayer(const LayerConfiguration_t& kLayerInfo);

    virtual eC_Bool UpdateLayerBuffers(
        const eC_UInt& uiLayerID,
        void* pkRenderBuffer,
        void* pkDisplayBuffer,
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight);

    virtual void UpdateLayer(const LayerConfiguration_t& rkLayerConfig, const LayerUpdate_t& eUpdateReason);

    virtual void EndOfRefresh();

    virtual eC_Bool SwapBuffers();

private:
    struct LayerData
    {
        LayerData() :
            BufferCount(0),
            RenderIndex(0),
            DisplayIndex(0),
            MemoryDC(NULL),
            DCBitmap(NULL),
            OriginalDCBitmap(NULL),
            Buffers(NULL)
        {
        }

        eC_UInt BufferCount;
        eC_UInt RenderIndex;
        eC_UInt DisplayIndex;
        HDC* MemoryDC; ///< Graphic Device Context, this is memory dc.
        HBITMAP* DCBitmap; ///< Windows' bitmap which is associated with m_hMemoryDC
        HBITMAP* OriginalDCBitmap; ///< Original bitmap created by windows by default
        BYTE** Buffers; ///< Pointer to memory of offscreen buffer
    };

private:
    void CreateScreens();
    void DeleteScreens();

    LPBITMAPINFO CreateBitmapInfoHeader(
        USHORT nBitsPerPixel,
        DWORD dwCompression,
        eC_UInt uiWidth,
        eC_UInt uiHeight);

    void CreateMemoryDC(
        LayerData*& kLayerData,
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight,
        const eC_UByte& ubBitsPerPixel,
        const eC_UInt& uiBufferCount);

    void RefreshLayer(const eC_UInt& uiLayerIndex);

    void RefreshLayers();

    CGfxEnvWin(const CGfxEnvWin& kSource);

    CGfxEnvWin& operator=(const CGfxEnvWin& kSource);

    BOOL OpenWindow(int nCmdShow);
    ATOM MyRegisterClass();
    static LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

private:
    HINSTANCE m_hInstance;

    /// Window which will be created by user application
    HWND m_hWnd;

    /// Graphic Device Context of actual windows' window on which the bitmap will be drawn.
    HDC m_hFrontBufferDC;

    HGLRC m_hOpenGLContext;    ///< Context for rendering commands in OpenGL

    eC_UInt m_uiDrawBufferIndex;
    eC_UInt m_uiDisplayBufferIndex;

    //LayerData m_kMainScreen;

    eC_UInt m_uiScreenCount;
    eC_TArray<LayerData*> m_akScreens;
};
#endif
