/**
 * @file max9867.c
 * @brief MAX9867 audio CODEC
 *
 * @section License
 *
 * Copyright (C) 2010-2015 Oryx Embedded SARL. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * @author Oryx Embedded SARL (www.oryx-embedded.com)
 * @version 1.6.0
 **/

//Dependencies
#include <stdio.h>
#include <string.h>
#include "bsp_api.h"
#include "m13_ra6m3_ek_audio.h"
#include "m13_ra6m3_ek_status.h"
#include "m13_ra6m3_ek_i2c.h"
#include "m13_ra6m3_ek_vcom.h"


/* RIIC driver handle */
bsp_i2c_handle_t audio_handle = BSP_I2C_INVALID_HANDLE;

/**
 * @brief MAX9867 audio CODEC initialization
 * @return Error code
 **/

bsp_status_t bsp_audio_init(void)
{
   bsp_status_t error;
   uint8_t dataRead;
   uint8_t value;


   // Debug message
   PRINTF_INFO("Initializing MAX9867...\r\n");

   // Retrieve I2C channel 0 handle
   audio_handle = bsp_i2c_open(BSP_I2C_CHANNEL_0);

   if (audio_handle == BSP_I2C_INVALID_HANDLE)
   {
       return BSP_STATUS_INVALID_CHANNEL;
   }

   //0x05 Configure system clock PSCLK between 10MHz and 20MHz & FREQ 0xA ou 0xB
   value = 0;
   value |= MAX9867_SYS_CLOCK_PSCLK_10_20MHZ;
   value |= MAX9867_SYS_CLOCK_FREQ_PLL_MODE;

   error = bsp_audio_write_reg(MAX9867_SYS_CLOCK, value);
   //Any error to report?
   if(error) return error;

   //0x06 & 0x07 Configure PLL & NI divider to 44.KHz
   error = bsp_audio_write_reg(MAX9867_STR_AUD_CLK_CTRL_H, 0x53);
   //Any error to report?
   if(error) return error;

   error = bsp_audio_write_reg(MAX9867_STR_AUD_CLK_CTRL_L, 0x5F);
   //Any error to report?
   if(error) return error;

   //0x08 Configure in Master Mode & 1 BCLK cycle delay
   value = 0;
   value |= MAX9867_DIG_AUD_IF1_MAS;
   value |= MAX9867_DIG_AUD_IF1_DLY;

   error = bsp_audio_write_reg(MAX9867_DIG_AUD_IF1, value);
   //Any error to report?
   if(error) return error;

   //0x09 Configure BSEL
   error = bsp_audio_write_reg(MAX9867_DIG_AUD_IF2, MAX9867_DIG_AUD_IF2_BSEL_48X_LRCLK );
   //Any error to report?
   if(error) return error;


   //0x10 Set Left Volume to Lowest
   error = bsp_audio_write_reg(MAX9867_LVOL_CTRL,0x14);
   //Any error to report?
   if(error) return error;

   //0x11 Set Right Volume to the Lowest
   error = bsp_audio_write_reg(MAX9867_RVOL_CTRL,0x14);
   //Any error to report?
   if(error) return error;

   //0x12 Set Left Microphone PreAmp to +20DB and Programmable Gain to +20dB
   value = 0;
   value |= MAX9867_MICL_GAIN_PALEN_20DB;
   value |= 0x09;

   error = bsp_audio_write_reg(MAX9867_MICL_GAIN, value);
   //Any error to report?
   if(error) return error;

   //0x14 Set Audio Input Mixer to Left/Right Analog Microphone
   error = bsp_audio_write_reg( MAX9867_ADC_IN, MAX9867_ADC_IN_MXINL_LR_AMIC );
   //Any error to report?
   if(error) return error;


   //0x16 Headphone mode in stereo single ended
   value = 0;
   value |= MAX9867_MODE_HPMODE_STER_SNGL_FST;
   value |= MAX9867_MODE_JDETEN;

   error = bsp_audio_write_reg(MAX9867_MODE, value);
   //Any error to report?
   if(error) return error;

   //0x17 enable left & right DAC + Left microphone & deactivate Right microphone
   value = 0;
   value |= MAX9867_SYS_SHDN_DALEN;
   value |= MAX9867_SYS_SHDN_DAREN;
   value |= MAX9867_SYS_SHDN_ADLEN;
   value &= ~MAX9867_SYS_SHDN_ADREN;

   error = bsp_audio_write_reg( MAX9867_SYS_SHDN, value);
   //Any error to report?
   if(error) return error;


   /*--- 0x17 Toogle SHDN bit whenever a configuration change is made ---*/

   //Read SYS_SHDN Regrister
   error = bsp_audio_read_reg(MAX9867_SYS_SHDN, &value); // dataRead
   //Any error to report?
   if(error) return error;

   //Enable SHDN bit

   // value must be value & ~MAX9867_SYS_SHDN_SHDN
   value &= 0x7F; //~MAX9867_SYS_SHDN_SHDN;

   error = bsp_audio_write_reg(MAX9867_SYS_SHDN , value);
   if(error) return error;

   value |= MAX9867_SYS_SHDN_SHDN;

   error = bsp_audio_write_reg(MAX9867_SYS_SHDN , value);
      if(error) return error;

   //Dump MAX9867 registers for debugging purpose
   bsp_audio_dump_reg();

   bsp_audio_read_reg(MAX9867_REVISION, &value);

   PRINTF_INFO(" MAX9867 revision is = 0x%02X \r\n", value);

   if (value == 0x42)
   {
       PRINTF_INFO(" Initializing MAX9867 success ! \r\n");
       return BSP_STATUS_OK;
   }
   else
   {
       PRINTF_INFO(" Initializing MAX9867 failure ! \r\n");
       return BSP_STATUS_INIT_FAILURE;
   }

   //Successful initialization
   return error;
}

bsp_status_t max9867Mute(void)
{
	bsp_status_t error;
	error = bsp_audio_write_reg(MAX9867_LVOL_CTRL, 0x3F);
	if(error) return error;
	error = bsp_audio_write_reg(MAX9867_RVOL_CTRL, 0x3F);
	if(error) return error;
	return error;
}

/**
 * @brief Adjust headphone volume
 * @param[in] left Left volume (should be in the range 0 to 100)
 * @param[in] right Right volume (should be in the range 0 to 100)
 * @return Error code
 **/

bsp_status_t bsp_audio_set_volume(uint8_t left, uint8_t right)
{
   bsp_status_t error;

   //Left volume
   if(left > 0x27)
   {
       left = 0x27;
   }

   //Right volume
   if(right > 0x27)
   {
       right = 0x27;
   }
   
   //Set headphone volume (left)
   error = bsp_audio_write_reg(MAX9867_LVOL_CTRL, left);
   //Any error to report?
   if(error) return error;

   //Set headphone volume (right)
   error = bsp_audio_write_reg(MAX9867_RVOL_CTRL, right);
   //Return status code
   return error;
}


/**
 * @brief Write MAX9867 register
 * @param[in] address Register address
 * @param[in] data Register value
 * @return Error code
 **/
bsp_status_t bsp_audio_write_reg(uint8_t address, uint8_t data)
{
    bsp_status_t error;

    error = bsp_i2c_write (audio_handle, BSP_I2C_SLAVE_ADDR_AUDIO, address, 1, &data, 1 );

    // Return status code
    return (error);
}

/**
 * @brief Write MAX9867 register
 * @param[in] address Register address
 * @param[in] data Register value
 * @return Error code
 **/
#if 0
bsp_status_t bsp_audio_write_reg16(uint8_t address, uint16_t data)
{
    uint16_t error = NO_ERROR_;
    uint32_t delay = 200000;
    uint8_t temp[2];

    temp[0] = (data>>8) & 0xFF ;
    temp[1] = data & 0xFF;

#if 0
    st_r_drv_riic_transfer_t i2c_write;
    i2c_write.device_address = BSP_I2C_SLAVE_ADDR_AUDIO;
    i2c_write.sub_address_type = RIIC_SUB_ADDR_WIDTH_8_BITS;
    i2c_write.sub_address = address;
    i2c_write.number_of_bytes = sizeof(temp);
    i2c_write.p_data_buffer = temp;

    if (DRV_ERROR >= control(handle_audio, CTL_RIIC_WRITE, &i2c_write))
    {
       error = ERROR_FAILURE_;

    }
#endif
    while (delay--)
    {
      __asm__("nop");
    }
    /* Return status code */
    return (error);

}
#endif
/**
 * @brief Read MAX9867 register
 * @param[in] address Register address
 * @param[out] data Register value
 * @return Error code
 **/
bsp_status_t bsp_audio_read_reg(uint8_t address, uint8_t *data)
{
    bsp_status_t error;

    error = bsp_i2c_read (audio_handle, BSP_I2C_SLAVE_ADDR_AUDIO, address, 1, data, 1 );

    // Return status code
    return (error);
}

/**
 * @brief Dump MAX9867 registers for debugging purpose
 * @return Error code
 **/

bsp_status_t bsp_audio_dump_reg(void)
{
	   bsp_status_t error;
	   uint8_t i;
	   uint8_t data;
	   //fsp_pack_version_t version = {0x00};

	   PRINTF_INFO("\r\n");
	   PRINTF_INFO("  Dump Start...\r\n");

	   //Loop through MAX9856 registers
	   for(i = 0; i <= 0x17; i++)
	   {
	      //Read register contents
	      error = bsp_audio_read_reg(i, &data);
	      //Any error to report?
	      if(error) break;

	      //Display current register
	      PRINTF_INFO("  0x%02X: 0x%02X \r\n", i, data);

	   }
	   	  //
	      error = bsp_audio_read_reg(MAX9867_REVISION_REV, &data);
	      //Any error to report?
	      if(!error)
	      {
	      //Display current register
	      PRINTF_INFO("  0x%02X: 0x%02X \r\n", 0xFF, data);
	      }

	   //Terminate with a line feed
	      PRINTF_INFO("\r\n");

	   //Return status code
	   return error;
}

