/**
 * @file m13_ra6m3_ek_lcd.c
 * @brief M13-RA6M3-EK LCD BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http:// www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

// Dependencies
#include "m13_ra6m3_ek_config.h"
#include "m13_ra6m3_ek_lcd.h"
#include "m13_ra6m3_ek_pinout.h"
#include "m13_ra6m3_ek_status.h"
#include "r_glcdc.h"
#include "FreeRTOS.h"
#include "common_data.h"
#include "r_display_api.h"

// Macro definition

// Display frame buffers
uint8_t lcd_fb_background [BSP_LCD_MAX_FRAME_BUFFER] [DISPLAY_BUFFER_STRIDE_BYTES_INPUT0 * 272] BSP_ALIGN_VARIABLE(64) BSP_PLACE_IN_SECTION(BSP_LCD_RAM_SECTION);

#if BSP_CONFIG_ENABLE_LCD_GRAPHIC_LAYER2
uint8_t lcd_fb_foreground [BSP_LCD_MAX_FRAME_BUFFER] [DISPLAY_BUFFER_STRIDE_BYTES_INPUT0 * 272] BSP_ALIGN_VARIABLE(64) BSP_PLACE_IN_SECTION(BSP_LCD_RAM_SECTION);
#endif

// Display device extended configuration
const glcdc_extended_cfg_t g_display_extend_cfg =
{
  .tcon_hsync                 = GLCDC_TCON_PIN_1,
  .tcon_vsync                 = GLCDC_TCON_PIN_2,
  .tcon_de                    = GLCDC_TCON_PIN_3,
  .correction_proc_order      = GLCDC_CORRECTION_PROC_ORDER_BRIGHTNESS_CONTRAST2GAMMA,
  .clksrc                     = GLCDC_CLK_SRC_INTERNAL,
  .clock_div_ratio            = GLCDC_PANEL_CLK_DIVISOR_32,
  .dithering_mode             = GLCDC_DITHERING_MODE_TRUNCATE,
  .dithering_pattern_A        = GLCDC_DITHERING_PATTERN_11,
  .dithering_pattern_B        = GLCDC_DITHERING_PATTERN_11,
  .dithering_pattern_C        = GLCDC_DITHERING_PATTERN_11,
  .dithering_pattern_D        = GLCDC_DITHERING_PATTERN_11
};

// Display control block instance
glcdc_instance_ctrl_t g_display_ctrl;

// Display interface configuration
const display_cfg_t g_display_cfg =
{
// Input1(Graphics1 layer) configuration
    .input[0] =
	{
    .p_base                   = (uint32_t *)&lcd_fb_background[0],
    .hsize                    = LCD_WIDTH,
    .vsize                    = LCD_HEIGHT,
    .hstride                  = DISPLAY_BUFFER_STRIDE_PIXELS_INPUT0,
    .format                   = DISPLAY_IN_FORMAT_16BITS_RGB565,
    .line_descending_enable   = false,
    .lines_repeat_enable      = false,
    .lines_repeat_times       = 0
    },
#if BSP_CONFIG_ENABLE_LCD_GRAPHIC_LAYER2
// Input2(Graphics2 layer) configuration
    .input[1] =
	{
	.p_base                   = (uint32_t *)&lcd_fb_foreground[0],
	.hsize                    = LCD_WIDTH,
	.vsize                    = LCD_HEIGHT,
	.hstride                  = DISPLAY_BUFFER_STRIDE_PIXELS_INPUT0,
	.format                   = DISPLAY_IN_FORMAT_16BITS_RGB565,
	.line_descending_enable   = false,
	.lines_repeat_enable      = false,
	.lines_repeat_times       = 0
	},
#endif

// Output configuration
    .output =
	{
		.htiming =
		{
		.total_cyc		      = 535,
        .display_cyc          = 480,
        .back_porch           = 43,
        .sync_width           = 4,
        .sync_polarity        = DISPLAY_SIGNAL_POLARITY_LOACTIVE
		},
		.vtiming =
		{
		.total_cyc            = 292,
		.display_cyc      	  = 272,
		.back_porch       	  = 12,
		.sync_width       	  = 4,
		.sync_polarity    	  = DISPLAY_SIGNAL_POLARITY_LOACTIVE
		},
		.format               = DISPLAY_OUT_FORMAT_24BITS_RGB888,
		.endian               = DISPLAY_ENDIAN_LITTLE,
		.color_order          = DISPLAY_COLOR_ORDER_BGR,
		.data_enable_polarity = DISPLAY_SIGNAL_POLARITY_HIACTIVE,
		.sync_edge            = DISPLAY_SIGNAL_SYNC_EDGE_RISING,
		.bg_color =
		{
		.byte 			      = { .a = 255, .r = 0, .g = 0, .b = 0 }
		},
		.dithering_on	      = false
	},

// Display device callback function pointer
	.p_callback			      = vsync_callback,
	.p_context			      = NULL,

// Display device extended configuration
	.p_extend			      = (void*) (&g_display_extend_cfg),
	.line_detect_ipl	      = (2),
	.underflow_1_ipl 	      = (BSP_IRQ_DISABLED),
	.underflow_2_ipl          = (BSP_IRQ_DISABLED),
	.line_detect_irq 	      = VECTOR_NUMBER_GLCDC_LINE_DETECT,
	.underflow_1_irq 	      = FSP_INVALID_VECTOR,
	.underflow_2_irq 	      = FSP_INVALID_VECTOR,

// Graphics layer 1 blend setting
    .layer[0] =
    {
    .coordinate              = { .x = 0, .y = 0, },
    .bg_color                =
    {
    .byte                    = { .a = 255, .r = 255, .g = 255, .b = 255 },
    },
    .fade_control            = DISPLAY_FADE_CONTROL_FADEIN,
    .fade_speed              = 0xFF,
    },

#if BSP_CONFIG_ENABLE_LCD_GRAPHIC_LAYER2
// Graphics layer 2 blend setting
    .layer[1] =
    {
    .coordinate              = { .x = 0, .y = 0, },
    .bg_color                =
    {
    .byte                    = { .a = 255, .r = 255, .g = 255, .b = 255 },
    },
    .fade_control            = DISPLAY_FADE_CONTROL_FADEIN,
    .fade_speed              = 0xFF,
    },
#endif
};

// Global variables
static ioport_instance_ctrl_t lcd_pin_ctrl;
static uint16_t *lcd_current_working_buffer = (uint16_t *) &lcd_fb_background[0];
static uint16_t background_color;
static uint16_t text_color;
static font_t * current_font;
uint16_t screen_rotation;
volatile uint32_t g_vsync_count = 0;

// Baclight IOPORT structure
static ioport_instance_ctrl_t backlight_pin_ctrl;

// Callback function for GLCDC vsync
void vsync_callback (display_callback_args_t * p_args)
{
    BaseType_t context_switch;
    xSemaphoreGiveFromISR(vsync_semaphore, &context_switch);
    portYIELD_FROM_ISR(context_switch);
}

// Wait until Vsync is triggered through callback function
void vsync_wait (void)
{
    xSemaphoreTake(vsync_semaphore, portMAX_DELAY);
}

/**
 * @name       bsp_lcd_io_init
 * @brief      Initialize / Configure LCD pins
 * @return     None
 */
void bsp_lcd_io_init (void)
{
    ioport_cfg_t lcd_pin_cfg;

    lcd_pin_cfg.number_of_pins = sizeof(m13_ra6m3_ek_lcd_pin_data)/sizeof(ioport_pin_cfg_t);

    lcd_pin_cfg.p_pin_cfg_data = &m13_ra6m3_ek_lcd_pin_data[0];

    R_IOPORT_Open(&lcd_pin_ctrl, &lcd_pin_cfg);
}

/**
 * @name       bsp_lcd_init
 * @brief      Initialize screen rotation and open GLCDC module
 * @param[in]  None
 * @return     None
 **/
void bsp_lcd_init (void)
{
    // Set screen rotation to default view
    screen_rotation = ROTATION_ZERO;

    // Open GLCDC module
    R_GLCDC_Open(&g_display_ctrl, &g_display_cfg);
}

/**
 * @name       bsp_lcd_enable_double_buffer
 * @brief      Set display buffer and working buffer
 * @param[in]  None
 * @return     None
 **/
void bsp_lcd_enable_double_buffer(void)
{
    bsp_lcd_set_display_buffer(1);
    bsp_lcd_set_working_buffer(2);
}

/**
 * @name       bsp_lcd_swap_buffer
 * @brief      Swap the display and working buffer based on the current buffer
 * @param[in]  None
 * @return     None
 **/
void bsp_lcd_swap_buffer(void)
{
    if( lcd_current_working_buffer == (uint16_t *) lcd_fb_background[0])
    {
        bsp_lcd_set_display_buffer(1);
        bsp_lcd_set_working_buffer(2);
    }
    else
    {
        bsp_lcd_set_display_buffer(2);
        bsp_lcd_set_working_buffer(1);
    }
}

/**
 * @name       bsp_lcd_set_display_buffer
 * @brief      Select display buffer
 * @param[in]  index : Buffer number
 * @return     None
 **/
void bsp_lcd_set_display_buffer(uint8_t index)
{
#if BSP_CONFIG_ENABLE_LCD_GRAPHIC_LAYER2

    // TODO

#else
    R_GLCDC_BufferChange(&g_display_ctrl, lcd_fb_background[index-1] ,DISPLAY_FRAME_LAYER_1);
#endif
}

/**
 * @name       bsp_lcd_set_working_buffer
 * @brief      Select current working buffer
 * @param[in]  index : Buffer number
 * @return     None
 **/
void bsp_lcd_set_working_buffer(uint8_t index)
{
    if(index >=1 && index <= BSP_LCD_MAX_FRAME_BUFFER)
    {
        lcd_current_working_buffer = (uint16_t *) lcd_fb_background[index-1];
    }
}

/**
 * @name       bsp_lcd_layer_fade
 * @brief      Initiate a fade on desired layer
 * @param[in]  disp_rt_cfg : Layer fade configuration
 * @param[in]  direction : Direction of the fade (fade-out or fade-in)
 * @param[in]  speed : Speed of the fade
 * @return     Return status
 **/
bsp_status_t bsp_lcd_layer_fade(display_runtime_cfg_t * disp_rt_cfg, uint8_t direction, uint16_t speed)
{
    // TODO
    return BSP_STATUS_OK;
}

/**
 * @name       bsp_lcd_display_on
 * @brief      Start Display
 * @param[in]  None
 * @return     None
 **/
void bsp_lcd_display_on (void)
{
    // Start GLCDC module
    R_GLCDC_Start(&g_display_ctrl);
}

 /**
 * @name       bsp_lcd_display_off
 * @brief      Stop Display
 * @param[in]  None
 * @return     None
 **/
void bsp_lcd_display_off (void)
{
    // Stop GLCDC module
    R_GLCDC_Stop(&g_display_ctrl);
}

/**
 * @name       bsp_lcd_clear
 * @brief      Clear LCD screen
 * @param[in]  color : Color in BGR565
 * @return     None
 **/
void bsp_lcd_clear (bsp_color_t color)
{
    for (uint32_t i = 0; i<( LCD_WIDTH * LCD_HEIGHT ); i++)
            {
                lcd_current_working_buffer[i] = color;
            }
}

/**
 * @name       bsp_lcd_set_backlight
 * @brief      Turn on LCD screen Backlight
 * @param[in]  backlight : Backlight level
 * @return     None
 **/
void bsp_lcd_set_backlight(bsp_io_level_t backlight)
{
    if (backlight == BSP_IO_LEVEL_LOW)
    {
        //Turn off led
        R_IOPORT_PinWrite(&backlight_pin_ctrl, LCD_BACKLIGHT_PIN, BSP_IO_LEVEL_LOW);
    }

    if (backlight == BSP_IO_LEVEL_HIGH)
    {
        //Turn on led
        R_IOPORT_PinWrite(&backlight_pin_ctrl, LCD_BACKLIGHT_PIN, BSP_IO_LEVEL_HIGH);
    }
}

/**
 * @name       bsp_lcd_read_pixel
 * @brief      Clear pixel
 * @param[in]  x : x coordinates
 * @param[in]  y : y coordinates
 * @return     Return pixel color
 **/
bsp_color_t bsp_lcd_read_pixel (uint16_t x, uint16_t y)
{
    bsp_color_t color;
	
    // Verify pixel is within LCD range
    if ( (x < LCD_WIDTH) && (y < LCD_HEIGHT) )
    {
        switch (screen_rotation)
        {
            case ROTATION_ZERO:
            {
                color = lcd_current_working_buffer[(y * LCD_WIDTH) + x];
                break;
            }
            case ROTATION_180:
            {
                color = lcd_current_working_buffer[((LCD_HEIGHT - y) * LCD_WIDTH) + (LCD_WIDTH - x)];
                break;
            }
            default:
            {
                color = lcd_current_working_buffer[(y * LCD_WIDTH) + x];
                break;
            }
        }
    }	
    return color;
}

/**
 * @name       bsp_lcd_draw_pixel
 * @brief      Draw pixel
 * @param[in]  x : x coordinates
 * @param[in]  y : y coordinates
 * @param[in]  color : Color in BGR565
 * @return none
 **/
void bsp_lcd_draw_pixel (uint16_t x, uint16_t y, bsp_color_t color)
{
    // Verify pixel is within LCD range
    if ( (x < LCD_WIDTH) && (y < LCD_HEIGHT) )
    {
        switch (screen_rotation)
        {
            case ROTATION_ZERO:
            {
                lcd_current_working_buffer[(y * LCD_WIDTH) + x] = color;
                break;
            }
            case ROTATION_180:
            {
                lcd_current_working_buffer[((LCD_HEIGHT - y) * LCD_WIDTH) + (LCD_WIDTH - x)] = color;
                break;
            }
            default:
            {
                lcd_current_working_buffer[(y * LCD_WIDTH) + x] = color;
                break;
            }
        }
    }
}

/**
 * @name       bsp_lcd_draw_horizontal_line
 * @brief      Draw horizontal line
 * @param[in]  x : x coordinates
 * @param[in]  y : y coordinates
 * @param[in]  length : Length of line
 * @param[in]  color : Color in BGR565
 * @return     None
 **/
void bsp_lcd_draw_horizontal_line (uint16_t x, uint16_t y, uint16_t length, bsp_color_t color)
{
    // Verify starting point of horizontal line is within LCD range
    if ( (x < LCD_WIDTH) && (y < LCD_HEIGHT) )
    {
        // If ending point is over LCD range, reduce line length to fit LCD range
        if((x+length) > LCD_WIDTH)
        {
            length = LCD_WIDTH - x;
        }
        // Draw line
        for(uint16_t i = 0; i <= length; i++)
        {
            bsp_lcd_draw_pixel(x, y, color);
            x++;
        }
    }
}

/**
 * @name       bsp_lcd_draw_vertical_line
 * @brief      Draw vertical line
 * @param[in]  x : x coordinates
 * @param[in]  y : y coordinates
 * @param[in]  length : Length of line
 * @param[in]  color : Color in BGR565
 * @return     None
 **/
void bsp_lcd_draw_vertical_line (uint16_t x, uint16_t y, uint16_t length, bsp_color_t color)
{
    // Verify starting point of vertical line is within LCD range
    if ( (x < LCD_WIDTH) && (y < LCD_HEIGHT) )
    {
        // If ending point is over LCD range, reduce line length to fit LCD range
        if((y+length) > LCD_HEIGHT)
        {
            length = LCD_HEIGHT - y;
        }
        // Draw line
        for(uint16_t i = 0; i < length; i++)
        {
            bsp_lcd_draw_pixel(x, y, color);
            y++;
        }
    }
}

/**
 * @name       bsp_lcd_draw_rect
 * @brief      Draw rectangle
 * @param[in]  x : x coordinates starting point
 * @param[in]  y : y coordinates starting point
 * @param[in]  width : rectangle width
 * @param[in]  height : rectangle height
 * @param[in]  length : Length of line
 * @param[in]  color : Color in BGR565
 * @return     None
 **/
void bsp_lcd_draw_rect(uint16_t x, uint16_t y, uint16_t width, uint16_t height, bsp_color_t color)
{
    // Verify starting point of vertical line is within LCD range
    if ( (x < LCD_WIDTH) && (y < LCD_HEIGHT) )
    {
        // If ending width point is over LCD range, reduce width to fit LCD range
        if((x+width) >= LCD_WIDTH)
        {
            width = LCD_WIDTH - x;
        }

        // If ending height point is over LCD range, reduce height to fit LCD range
        if((y+height) >= LCD_HEIGHT)
        {
            height = LCD_HEIGHT - y;
        }

        // Draw horizontal line 1
        bsp_lcd_draw_horizontal_line(x, y, width, color);

        // Draw horizontal line 2
        bsp_lcd_draw_horizontal_line(x, (y+height), width, color);

        // Draw vertical line 1
        bsp_lcd_draw_vertical_line(x, y, height, color);

        // Draw vertical line 2
        bsp_lcd_draw_vertical_line((x+width), y, height, color);
    }
}

/**
 * @name       bsp_lcd_fill_rect 
 * @brief      Fill rectangle
 * @param[in]  x : x coordinates starting point
 * @param[in]  y : y coordinates starting point
 * @param[in]  width : rectangle width
 * @param[in]  height : rectangle height
 * @param[in]  length : Length of line
 * @param[in]  color : Color in BGR565
 * @return     None
 **/
void bsp_lcd_fill_rect(uint16_t x, uint16_t y, uint16_t width, uint16_t height, bsp_color_t color)
{
    // Verify starting point of vertical line is within LCD range
    if ( (x < LCD_WIDTH) && (y < LCD_HEIGHT) )
    {
        // If ending width point is over LCD range, reduce width to fit LCD range
        if((x+width) >= LCD_WIDTH)
        {
            width = LCD_WIDTH - x;
        }

        // If ending height point is over LCD range, reduce height to fit LCD range
        if((y+height) >= LCD_HEIGHT)
        {
            height = LCD_HEIGHT - y;
        }
        // Fill rectangle with height number of lines
        for(uint16_t i = 0; i < height; i++)
        {
            bsp_lcd_draw_horizontal_line(x,y,width, color);
            y++;
        }
    }
}

/**
 * @name       bsp_lcd_set_background_color 
 * @brief      Set color for non-displayed pixel in font
 * @param[in]  color : Color in BGR565
 * @return     None
 **/
void bsp_lcd_set_background_color (bsp_color_t color)
{
    background_color = color;
}

/**
 * @name       bsp_lcd_set_text_color 
 * @brief      Set color for displayed pixel in font
 * @param[in]  color : Color in BGR565
 * @return     None
 **/
void bsp_lcd_set_text_color (bsp_color_t color)
{
    text_color = color;
}

/**
 * @name       bsp_lcd_set_font 
 * @brief      Select current font
 * @param[in]  p_font : Pointer to font structure
 * @return     None
 **/
void bsp_lcd_set_font(font_t *p_font)
{
    current_font = p_font;
}

/**
 * @name       bsp_lcd_display_char 
 * @brief      Display character at given coordinates
 * @param[in]  x : x coordinates starting point
 * @param[in]  y : y coordinates starting point
 * @param[in]  c : Character to display
 * @return none
 **/
void bsp_lcd_display_char (uint16_t x, uint16_t y, uint8_t c)
{
    uint16_t index, color, char_width;
    uint16_t pixel_state;

    // Verify character is between 0x20 et 0x7E
    if ( c>=32 && c<=126 )
    {
        char_width = (current_font->width + 7) / 8;

        // Position the font index to the desired character
        index = (c-32) * char_width * current_font->height;

        // Go through each of the 8 lines of the character
        for(uint8_t j = 0; j<current_font->height; j++)
        {
            // Draw each of the X pixels of the character line
            for(uint8_t i = 0; i <= (current_font->width); i++)
            {
                // Check which pixel is to be drawn or not and choose color for each of them
                pixel_state = current_font->table [ index + (char_width*j) + (i/8) ] & (0x80>>(i%8));

                if(pixel_state)
                {
                    color = text_color;
                }
                else
                {
                    color = background_color;
                }

                // Draw each pixel
                bsp_lcd_draw_pixel(x+i, y+j, color);
            }
        }
    }
}

/**
 * @name       bsp_lcd_display_string 
 * @brief      Display character string at given coordinates
 * @param[in]  x : x coordinates starting point
 * @param[in]  y : y coordinates starting point
 * @param[in]  *s : Character string to display
 * @return     None
 **/
void bsp_lcd_display_string (uint16_t x, uint16_t y, const char *s)
{
    while (*s != '\0')
    {
        bsp_lcd_display_char(x , y , *s);
        x+=current_font->width;
        s++;
    }
}

/* END OF FILE */
