/**
 * @file m13_ra6m3_ek_led.c
 * @brief M13-RA6M3-EK led BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#include "r_qspi.h"
#include "m13_ra6m3_ek_config.h"
#include "m13_ra6m3_ek_pinout.h"
#include "m13_ra6m3_ek_status.h"
#include "m13_ra6m3_ek_qspi.h"
#include "m13_ra6m3_ek_vcom.h"
#include "debug.h"

// Global variables
static ioport_instance_ctrl_t qspi_pin_ctrl;

// Macro definition


qspi_instance_ctrl_t qspi_ctrl;

static const spi_flash_erase_command_t qspi_erase_command_list[] =
{
#if 4096 > 0
		{ .command = BSP_QSPI_SE, .size = 4096 },
#endif
#if 32768 > 0
		{ .command = BSP_QSPI_BE32K, .size = 32768 },
#endif
#if 65536 > 0
		{ .command = BSP_QSPI_BE, .size = 65536 },
#endif
#if 0xC7 > 0
		{ .command = BSP_QSPI_CE, .size = SPI_FLASH_ERASE_SIZE_CHIP_ERASE },
#endif
};

static const qspi_extended_cfg_t qspi_extended_cfg =
{
    .min_qssl_deselect_cycles   = QSPI_QSSL_MIN_HIGH_LEVEL_8_QSPCLK,
    .qspclk_div                 = QSPI_QSPCLK_DIV_2,
};

spi_flash_cfg_t qspi_cfg =
{
#if BSP_CONFIG_ENABLE_QPI_MODE
	.spi_protocol               = SPI_FLASH_PROTOCOL_QPI,
#else
	.spi_protocol               = SPI_FLASH_PROTOCOL_EXTENDED_SPI,
#endif
	.read_mode                  = SPI_FLASH_READ_MODE_FAST_READ_QUAD_IO,
	.address_bytes              = SPI_FLASH_ADDRESS_BYTES_3,
	.dummy_clocks               = SPI_FLASH_DUMMY_CLOCKS_DEFAULT,
	.page_program_address_lines = SPI_FLASH_DATA_LINES_1,
	.page_size_bytes            = 256,
	.page_program_command       = BSP_QSPI_PAGE_PROG,
	.write_enable_command       = BSP_QSPI_WRITE_ENABLE,
	.status_command             = BSP_QSPI_READ_STATUS_REG,
	.write_status_bit           = 1,
	.xip_enter_command          = 0x20,
	.xip_exit_command           = 0xFF,
	.p_erase_command_list       = &qspi_erase_command_list[0],
	.erase_command_list_length  = sizeof(qspi_erase_command_list) / sizeof(qspi_erase_command_list[0]),
	.p_extend                   = &qspi_extended_cfg,
};


/**
 * @name       bsp_qspi_io_init
 * @brief      Initialize ADC pins
 * @return     None
 **/
void bsp_qspi_io_init (void)
{
    ioport_cfg_t qspi_pin_cfg;

    // Load pin configuration from pin table
    qspi_pin_cfg.number_of_pins = sizeof(m13_ra6m3_ek_qspi_pin_data)/sizeof(ioport_pin_cfg_t);
    qspi_pin_cfg.p_pin_cfg_data = &m13_ra6m3_ek_qspi_pin_data[0];

    R_IOPORT_Open(&qspi_pin_ctrl, &qspi_pin_cfg);
}

/**
 * @name       bsp_qspi_init
 * @brief      Call QSPI pin configuration function, Open the QSPI Instance and Write Status and Configuration Registers
 * @param[in]  None
 * @return     Error code (see m13_ra6m3_ek_status.h for enumeration)
 **/
bsp_status_t bsp_qspi_init (void)
{
    fsp_err_t error = FSP_SUCCESS;
    bsp_status_t status;
    uint8_t   data[3];

    // Configure the board QSPI pins
    bsp_qspi_io_init();

    // Check which SPI Mode is set in the configuration

    // If QPI mode is selected
    if(qspi_cfg.spi_protocol == SPI_FLASH_PROTOCOL_QPI)
    {
    	// Re-write SPI protocol to Extended SPI
    	qspi_cfg.spi_protocol = SPI_FLASH_PROTOCOL_EXTENDED_SPI;

    	// Open QSPI Instance
    	error = R_QSPI_Open(&qspi_ctrl, &qspi_cfg);

        if (error != FSP_SUCCESS)
        {
            return BSP_STATUS_OPEN_FAILURE;
        }

    	// Set Read mode in any Dual/Quad Read mode. (DREAD, 2READ, QREAD mode can only be done while in SPI mode)
    	bsp_qspi_set_read_mode(BSP_QSPI_4DTRD);

    	// Set QPI mode This can only be done in SPI mode
    	bsp_qspi_set_qpi_mode();

    	// Close the QSPI Instance
    	R_QSPI_Close(&qspi_ctrl);

    	// Re-write SPI protocol to QPI mode
    	qspi_cfg.spi_protocol = SPI_FLASH_PROTOCOL_QPI;

    	// Re-open QSPI Instance in QPI mode
    	error = R_QSPI_Open(&qspi_ctrl, &qspi_cfg);

        if (error != FSP_SUCCESS)
        {
            return BSP_STATUS_OPEN_FAILURE;
        }
        else
        {
        	PRINTF_INFO(" QPI mode activated and... ");
        }
    }
    // If Extended SPI is selected
    else
    {
    	// Re-write SPI protocol to QPI mode
    	qspi_cfg.spi_protocol = SPI_FLASH_PROTOCOL_QPI;

    	// Open QSPI Instance
    	error = R_QSPI_Open(&qspi_ctrl, &qspi_cfg);

    	// Reset the QPI mode, this can only be done while in QPI mode
    	bsp_qspi_reset_qpi_mode();

    	// Close the QSPI Instance
    	error = R_QSPI_Close(&qspi_ctrl);

    	// Re-write SPI protocol to Extended SPI
    	qspi_cfg.spi_protocol = SPI_FLASH_PROTOCOL_EXTENDED_SPI;

    	// Re-open QSPI Instance in Extended SPI mode
    	error = R_QSPI_Open(&qspi_ctrl, &qspi_cfg);

    	// Set Read mode to FAST_READ
    	bsp_qspi_set_read_mode(BSP_QSPI_FAST_READ);

    	PRINTF_INFO(" QPI mode deactivated and... ");
    }

    // Write one byte to enable writing to the status register
    data[0] = BSP_QSPI_WRITE_ENABLE;
    error = R_QSPI_DirectWrite(&qspi_ctrl, &data[0], 1, false);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_QSPI_WRITE_FAILURE;
    }
    // Write Status Register followed by QSPI Status Register and Configuration Register values
    data[0] = BSP_QSPI_WRITE_STATUS_REG;
    data[1] = BSP_QSPI_STATUS_REG;
    data[2] = BSP_QSPI_CONFIG_REG;

    error   = R_QSPI_DirectWrite(&qspi_ctrl, &data[0], 3, false);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_QSPI_WRITE_FAILURE;
    }

	bsp_qspi_get_status();

    // Check flash device ID
    status = bsp_qspi_id_read();

    if (status != BSP_STATUS_OK)
    {
    	PRINTF_INFO(" QSPI Init Failed!\r\n");
    }
    else
    {
        PRINTF_INFO(" QSPI Init Success!\r\n");
    }

    return BSP_STATUS_OK;
}

/**
 * @name       bsp_qspi_set_qpi_mode
 * @brief      Set QPI mode in flash device and MCU
 * @param[in]  None
 * @return     Error code (see m13_ra6m3_ek_status.h for enumeration)
 **/
bsp_status_t bsp_qspi_set_qpi_mode (void)
{
    fsp_err_t error = FSP_SUCCESS;
    uint8_t data;
    uint8_t manufacturer_id;
    uint16_t device_id;

    // Enable writing
    data = BSP_QSPI_WRITE_ENABLE;
    error = R_QSPI_DirectWrite(&qspi_ctrl, &data, 1, false);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_QSPI_WRITE_FAILURE;
    }

    // Write the QPI instruction to enable QPI mode
    data = BSP_QSPI_ENABLE_QPI;
    error   = R_QSPI_DirectWrite(&qspi_ctrl, &data, 1, false);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_QSPI_WRITE_FAILURE;
    }

    // Enable writing
    data = BSP_QSPI_WRITE_ENABLE;
    error = R_QSPI_DirectWrite(&qspi_ctrl, &data, 1, false);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_QSPI_WRITE_FAILURE;
    }

    // Set the QPI protocol in MCU
    error = R_QSPI_SpiProtocolSet(&qspi_ctrl, SPI_FLASH_PROTOCOL_QPI);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_QSPI_FAILURE;
    }
    else
    {
    	return BSP_STATUS_OK;
    }
}

/**
 * @name       bsp_qspi_reset_qpi_mode
 * @brief      Disable the Flash device QPI mode and returns to SPI mode. Can only be called while the QSPI Instance is opened in QPI mode
 * @param[in]  None
 * @return     Error code (see m13_ra6m3_ek_status.h for enumeration)
 **/
bsp_status_t bsp_qspi_reset_qpi_mode (void)
{
    fsp_err_t error = FSP_SUCCESS;
    uint8_t data;
    uint8_t manufacturer_id;
    uint16_t device_id;

    // Enable writing
    data = BSP_QSPI_WRITE_ENABLE;
    error = R_QSPI_DirectWrite(&qspi_ctrl, &data, 1, false);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_QSPI_WRITE_FAILURE;
    }

    // Write the Reset QPI instruction to disable QPI mode
    data = BSP_QSPI_DISABLE_QPI;
    error   = R_QSPI_DirectWrite(&qspi_ctrl, &data, 1, false);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_QSPI_WRITE_FAILURE;
    }

    // Set the SPI protocol in MCU
    error = R_QSPI_SpiProtocolSet(&qspi_ctrl, SPI_FLASH_PROTOCOL_EXTENDED_SPI);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_QSPI_FAILURE;
    }
    else
    {
    	return BSP_STATUS_OK;
    }
}

/**
 * @name       bsp_qspi_set_read_mode
 * @brief      Enable the Flash device QPI mode. Can only be called while the QSPI Instance is opened in SPI mode
 * @param[in]  None
 * @return     Error code (see m13_ra6m3_ek_status.h for enumeration)
 **/
bsp_status_t bsp_qspi_set_read_mode (bsp_qspi_read_mode read_mode)
{
    fsp_err_t error = FSP_SUCCESS;
    uint8_t data;

    // Enable writing
    data = BSP_QSPI_WRITE_ENABLE;
    error = R_QSPI_DirectWrite(&qspi_ctrl, &data, 1, false);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_QSPI_WRITE_FAILURE;
    }

    // Write the Reset QPI instruction to disable QPI mode
    data = read_mode;
    error   = R_QSPI_DirectWrite(&qspi_ctrl, &data, 1, false);

    if (error != FSP_SUCCESS)
    {
        return BSP_STATUS_QSPI_FAILURE;
    }
    else
    {
    	return BSP_STATUS_OK;
    }
}

/**
 * @name       bsp_qspi_id_read
 * @brief      Read the flash device ID registers and check if Macronix MX25L25645G device is recognized
 * @param[in]  None
 * @return     Error code (see m13_ra6m3_ek_status.h for enumeration)
 **/
bsp_status_t bsp_qspi_id_read (void)
{
    fsp_err_t error = FSP_SUCCESS;
    uint8_t data[3];
    uint8_t manufacturer_id;
    uint16_t device_id;

    // Write the RDID or QPIID instruction base on the SPI protocol in the configuration
    if (qspi_cfg.spi_protocol != SPI_FLASH_PROTOCOL_EXTENDED_SPI )
    {
    	// Sent QPIID instruction
        data[0] = BSP_QSPI_QPIID;
    }
    else
    {
    	// Sent RDID instruction
        data[0] = BSP_QSPI_READ_ID;
    }

    // Send ID read instruction to Flash device
    error   = R_QSPI_DirectWrite(&qspi_ctrl, &data[0], 1, true);

    // Read the 3 ID bytes
    error = R_QSPI_DirectRead(&qspi_ctrl, &data[0], 3);

    bsp_qspi_get_status();

    // Format the data before printing
    manufacturer_id = data[0];
    device_id = (data[1] << 8) + data[2];

    if ((manufacturer_id != BSP_QSPI_MFG_ID) || (device_id != BSP_QSPI_DEVICE_ID) )
    {
        PRINTF_INFO(" Unknown SPI flash device\r\n");
        PRINTF_INFO(" Vendor      : 0x%02X \r\n", manufacturer_id);
        PRINTF_INFO(" Device ID   : 0x%04X \r\n", device_id);

        return BSP_STATUS_QSPI_ID_READ_FAILURE;
    }
    else if ( (manufacturer_id == BSP_QSPI_MFG_ID) && (device_id == BSP_QSPI_DEVICE_ID) )
    {
    	//PRINTF_INFO(" Macronix MX25L25645G Device detected \r\n");

    	return BSP_STATUS_OK;
    }
    else
    {
    	return BSP_STATUS_OK;
    }
}

/**
 * @name       bsp_qspi_read_status_reg
 * @brief      Read the flash device's Status and Configuration Registers and displays the values in the VCOM
 * @param[in]  None
 * @return     Error code (see m13_ra6m3_ek_status.h for enumeration)
 **/
void bsp_qspi_read_status_reg (void)
{
    fsp_err_t error = FSP_SUCCESS;
    uint8_t data;

    PRINTF_INFO("\r\n ***** STATUS REG VALUE *****\r\n");

    // Write the Read Status Register instruction
    data = BSP_QSPI_READ_STATUS_REG;
    error = R_QSPI_DirectWrite(&qspi_ctrl, &data, 1, true);

    // Read 1 byte to get the Status Register value
    error = R_QSPI_DirectRead(&qspi_ctrl, &data, 1);

    bsp_qspi_get_status();

    PRINTF_INFO(" Status Registor  : 0x%02X \r\n", data);

    // Write the Read Configuration Register instruction
    data = BSP_QSPI_READ_CONFIG_REG;
    error   = R_QSPI_DirectWrite(&qspi_ctrl, &data, 1, true);

    // Read 1 byte to get the Configuration Register value
    error = R_QSPI_DirectRead(&qspi_ctrl, &data, 1);

    bsp_qspi_get_status();

    PRINTF_INFO(" Config Registor  : 0x%02X \r\n", data);
}

/**
 * @name       bsp_qspi_test
 * @brief      For test purpose only. Erase then write data in first sector, read it and compare it
 * @param[in]  None
 * @return     Error code (see m13_ra6m3_ek_status.h for enumeration)
 **/
bsp_status_t bsp_qspi_test(void)
{
   uint8_t   *p_qspi_addr;
   uint8_t   write_buffer [BSP_QSPI_PAGE_WRITE_SIZE] = {0x00,};
   uint8_t   read_buffer  [BSP_QSPI_PAGE_WRITE_SIZE] = {0x00,};
   fsp_err_t error = FSP_SUCCESS;

   //Debug message
   PRINTF_INFO("\r\n ** Testing QSPI device ...\r\n");

   //Point to the base address of the QSPI
   p_qspi_addr = (uint8_t *) BSP_QSPI_DEVICE_BASE_ADDRESS;

   // Check if QPI mode is selected in configuration and set QPI mode in flash device and MCU
   if(qspi_cfg.spi_protocol == SPI_FLASH_PROTOCOL_QPI)
   {
	   bsp_qspi_set_qpi_mode();
   }

   // Fill the write buffer to be written in QSPI flash device with a pattern
   for (uint8_t write_index = 0; write_index < BSP_QSPI_PAGE_WRITE_SIZE; write_index++)
   {
	   write_buffer[write_index] = write_index % 17;
   }

#if 0
   // Print Write buffer for debug purpose
   PRINTF_INFO("\r\n Write buffer value... \r\n");
   TRACE_INFO_ARRAY("  ", write_buffer, sizeof(write_buffer));

   // Erase flash device for one sector
   error = R_QSPI_Erase(&qspi_ctrl, p_qspi_addr, BSP_QSPI_SECTOR_SIZE);

   if (error != FSP_SUCCESS)
   {
	   PRINTF_INFO(" QSPI erase failure ...\r\n");
       return BSP_STATUS_QSPI_ERASE_FAILURE;
   }
   else
   {
	   PRINTF_INFO(" <QSPI erase success ...\r\n");
	   bsp_qspi_get_status();
   }

   // Verify erase
   for(uint16_t qspi_mem_index = 0; qspi_mem_index < BSP_QSPI_SECTOR_SIZE  ; qspi_mem_index++)
   {
	   if(BSP_QSPI_DEFAULT_MEM_VAL != p_qspi_addr[qspi_mem_index])
	   {
		   PRINTF_INFO(" !! Erase verify failure at address = 0x%08X !!\r\n", (p_qspi_addr + qspi_mem_index) );
		   return BSP_STATUS_QSPI_ERASE_FAILURE;
	   }

	   if(qspi_mem_index < BSP_QSPI_PAGE_WRITE_SIZE)
	   read_buffer[qspi_mem_index] = p_qspi_addr[qspi_mem_index];

   }

   PRINTF_INFO("\r\n Sector 0 value after erase : \r\n");
   TRACE_INFO_ARRAY("  ", read_buffer, sizeof(read_buffer));

   // Write data to flash device
   error = R_QSPI_Write(&qspi_ctrl, write_buffer, (uint8_t *)BSP_QSPI_FLASH_ADDRESS(BSP_QSPI_PAGE_FIRST), BSP_QSPI_PAGE_WRITE_SIZE);

   if (error != FSP_SUCCESS)
   {
	   PRINTF_INFO("\r\n QSPI write failure ...\r\n");
       return BSP_STATUS_QSPI_WRITE_FAILURE;
   }
   else
   {
	   bsp_qspi_get_status();
   }
#endif

   // Read data from QSPI memory region
   memcpy(read_buffer, (uint8_t *)BSP_QSPI_FLASH_ADDRESS(BSP_QSPI_PAGE_FIRST), BSP_QSPI_PAGE_WRITE_SIZE);

   // Verify the read data match the written data
   //if ( BSP_STATUS_OK == (bsp_status_t)(memcmp(read_buffer, write_buffer, BSP_QSPI_PAGE_WRITE_SIZE)) )
   {
	   PRINTF_INFO("\r\n Read buffer value after write... \r\n");
	   PRINTF_ARRAY("  ", read_buffer, sizeof(read_buffer));

	   PRINTF_INFO("\r\n QSPI test success ...\r\n");


	   return BSP_STATUS_OK;
   }
#if 0
   else
   {
	   PRINTF_INFO("\r\n !! QSPI test failure ... !!\r\n");
	   return BSP_STATUS_QSPI_FAILURE;
   }
#endif
}

/**
 * @name       bsp_qspi_get_status
 * @brief      Wait for QSPI flash device status register to get idle till operation is in progress
 * @return     Error code (see m13_ra6m3_ek_status.h for enumeration)
 **/
bsp_status_t bsp_qspi_get_status(void)
{
    spi_flash_status_t status = {.write_in_progress = true};
    int16_t time_out          = 20000;
    fsp_err_t error           = FSP_SUCCESS;

    // Wait for status register to update
    do
    {
    	error = R_QSPI_StatusGet(&qspi_ctrl, &status);

    	--time_out;

        if ( 0 >= time_out)
        {
        	PRINTF_INFO("\r\n !! Timeout : No result from QSPI flash status register !! \r\n");
            return BSP_STATUS_TIME_OUT;
        }

    }while (false != status.write_in_progress);

    return BSP_STATUS_OK;
}
