/**
 * @file m13_ra6m3_ek_qspi.h
 * @brief m13_ra6m3_ek QSPI BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#ifndef _M13_RA6M3_EK_QSPI_H
#define _M13_RA6M3_EK_QSPI_H

// Dependencies
//#include "r_ioport.h"
#include "m13_ra6m3_ek_status.h"

// macro definitions
#define BSP_QSPI_DEVICE_SIZE               32 * 1024 * 1024
#define BSP_QSPI_DEVICE_BASE_ADDRESS       0x60000000
#define BSP_QSPI_FLASH_ADDRESS(page_no)    (uint8_t *) (BSP_QSPI_DEVICE_BASE_ADDRESS + (page_no * BSP_QSPI_PAGE_WRITE_SIZE))
#define BSP_QSPI_PAGE_FIRST                0U
#define BSP_QSPI_PAGE_WRITE_SIZE           255U
#define BSP_QSPI_SECTOR_SIZE               4096U
#define BSP_QSPI_32K_BLOCK_SIZE            32768
#define BSP_QSPI_64K_BLOCK_SIZE            65536U
#define BSP_QSPI_DEFAULT_MEM_VAL           0xFF

#define BSP_QSPI_WRITE_ENABLE              0x06   // Write Enable
#define BSP_QSPI_WRITE_DESABLE             0x04   // Write Disable

#define BSP_QSPI_READ_ID                   0x9F   // RDID  : in SPI mode, Read the flash MFG ID and Device ID
#define BSP_QSPI_QPIID                     0xAF   // QPIID : In QPI mode, Read the flash MFG ID and Device ID

#define BSP_QSPI_READ_STATUS_REG           0x05   // RDSR  : Read Status register
#define BSP_QSPI_READ_CONFIG_REG           0x15   // RDCR  : Read Configuration register

#define BSP_QSPI_WRITE_STATUS_REG          0x01   // WDSR  : Write Status register
#define BSP_QSPI_STATUS_REG                0x40   // Status Register bit6 is set. Quad Enabled for Status register : QE = 1
#define BSP_QSPI_CONFIG_REG                0x00   // Config Register is set to default value

#define BSP_QSPI_SE                        0x20   // SE     : Sector Erase
#define BSP_QSPI_BE32K                     0x52   // BE32K  : 32K-byte block erase
#define BSP_QSPI_BE                        0xD8   // BE     : 64K-byte block erase
#define BSP_QSPI_CE                        0xC7   // CE     : Chip Erase 60h or C7h

#define BSP_QSPI_PAGE_PROG                 0x02   // PP     : Page program
#define BSP_QSPI_4PP                       0x38   // 4PP    : Quad Page program
#define BSP_QSPI_DP                        0xB9   // DP     : Deep Power Down

#define BSP_QSPI_ENABLE_QPI                0x35   // Enable QPI Mode
#define BSP_QSPI_DISABLE_QPI               0xF5   // Reset QPI Mode

#define BSP_QSPI_MFG_ID                    0xC2   // MX25L25645G Manufacturer ID is C2
#define BSP_QSPI_DEVICE_ID                 0x2019 // MX25L25645G Device ID is 2019

// Typedef definition
typedef enum
{
	BSP_QSPI_READ      = 0x03,   // Read instruction
	BSP_QSPI_FAST_READ = 0x0B,   // Fast-Read instruction
	BSP_QSPI_DREAD     = 0x3B,   // Dual Output Read
	BSP_QSPI_2READ     = 0xBB,   // Dual I/O Read
	BSP_QSPI_QREAD     = 0x6B,   // Quad Output Read
	BSP_QSPI_4READ     = 0xEB,   // Quad I/O Read
    BSP_QSPI_4DTRD     = 0xED,   // Double Transfer Rate Quad I/O Read
}bsp_qspi_read_mode;

// BSP Functions
void         bsp_qspi_io_init (void);
bsp_status_t bsp_qspi_init (void);

bsp_status_t bsp_qspi_set_qpi_mode (void);
bsp_status_t bsp_qspi_reset_qpi_mode (void);
bsp_status_t bsp_qspi_set_read_mode (bsp_qspi_read_mode read_mode);

bsp_status_t bsp_qspi_id_read (void);
void         bsp_qspi_read_status_reg (void);
bsp_status_t bsp_qspi_test(void);
bsp_status_t bsp_qspi_get_status(void);

#endif

