/**
 * @file m13_ra6m3_ek_touch.c
 * @brief M13-RA6M3-EK touchscreen BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http:// www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

// Dependencies
#include "m13_ra6m3_ek_touch.h"
#include "m13_ra6m3_ek_vcom.h"
#include "r_ioport.h"
#include "common_data.h"
#include "r_icu.h"

// Macro definitions

// Global variables
int32_t touch_read_request = 0;
bsp_i2c_handle_t touch_handle;

// IRQ touch configuration

void touch_irq_cb(external_irq_callback_args_t *p_args);

icu_instance_ctrl_t irq6_touch_ctrl;

// IRQ6 configuration
const external_irq_cfg_t irq6_touch_cfg =
{
  .channel       = 6,
  .trigger       = EXTERNAL_IRQ_TRIG_FALLING,
  .filter_enable = true,
  .pclk_div      = EXTERNAL_IRQ_PCLK_DIV_BY_64,
  .p_callback    = touch_irq_cb,
  .p_context     = NULL,
  .p_extend      = NULL,
  .ipl           = (15),
  .irq           = VECTOR_NUMBER_ICU_IRQ6,
};

// IRQ6 interface module
const external_irq_instance_t irq6_touch =
{
    .p_ctrl      = &irq6_touch_ctrl,
    .p_cfg       = &irq6_touch_cfg,
    .p_api       = &g_external_irq_on_icu
};

/**
 * @name       bsp_touch_init
 * @brief      Configure Touch IRQ and Touchscreen in Normal Mode
 * @param[in]  None
 * @return     Error code
 */
bsp_status_t bsp_touch_init(void)
{
    uint8_t value;
    bsp_status_t error;
    ioport_instance_ctrl_t irq6_pin_ctrl;
    ioport_cfg_t irq6_pin_cfg;

    // configure touch IRQ pin


    irq6_pin_cfg.number_of_pins = sizeof(m13_ra6m3_ek_irq6_pin_data)/sizeof(ioport_pin_cfg_t);
    irq6_pin_cfg.p_pin_cfg_data = &m13_ra6m3_ek_irq6_pin_data[0];

    //Configure led pins
    R_IOPORT_Open(&irq6_pin_ctrl, &irq6_pin_cfg);


    // Open I2C channel 3 and initialize Touch handler
    //touch_handle = bsp_i2c_open(BSP_I2C_CHANNEL_3);
    touch_handle = bsp_i2c_open(BSP_I2C_CHANNEL_0);

    if (touch_handle == BSP_I2C_INVALID_HANDLE)
    {
        return BSP_STATUS_INVALID_CHANNEL;
    }

    // Activate touch interrupt as IRQ6
    R_ICU_ExternalIrqOpen(irq6_touch.p_ctrl, irq6_touch.p_cfg);
    R_ICU_ExternalIrqEnable(irq6_touch.p_ctrl);

    // Force Touch in Normal Mode
    error = bsp_touch_write_reg(FT5206_DEVICE_MODE, FT5206_DEVICE_MODE_MODE_NORMAL);

    // If Touch Normal Mode could not be written, return error code
    if(error == BSP_STATUS_FAILURE)
    {
        PRINTF_INFO(" Error could not write Touch Mode \r\n");
        return error;
    }

    // Check device mode
    bsp_touch_read_reg(FT5206_DEVICE_MODE, &value);

    // Return error code if wrong device mode detected
    if(value != FT5206_DEVICE_MODE_MODE_NORMAL)
    {
        PRINTF_INFO(" Wrong device mode = 0x%02X instead of 0x00 \r\n", value);
        return BSP_STATUS_INVALID_VERSION;
    }

//    PRINTF_INFO(" Device mode = 0x%02X \r\n", value);
    PRINTF_INFO(" Touch Init Success! \r\n");

    return BSP_STATUS_OK;
}

/**
 * @name       bsp_get_touch_status
 * @brief      Check Touch Status Register
 * @param[in]  touch_status
 * @return     Error code
 */
bsp_status_t bsp_touch_get_status(uint8_t *touch_status)
{
    bsp_status_t error;

    error = bsp_i2c_read (touch_handle, BSP_I2C_SLAVE_ADDR_TOUCH, FT5206_TD_STATUS, 1, touch_status, 1);

    return error;
}

/**
 * @name       bsp_get_gest_id
 * @brief      Check gesture after a valid touch is detected
 * @param[in]  touch_status
 * @return     Error code
 */
bsp_status_t bsp_touch_get_gest_id(uint8_t *gest_id)
{
    bsp_status_t error;

    error = bsp_i2c_read (touch_handle, BSP_I2C_SLAVE_ADDR_TOUCH, FT5206_GEST_ID, 1, gest_id, sizeof(gest_id) );

    return error;
}

/**
 * @name       bsp_touch_get_data
 * @brief      Retrieve all the touch data from registers
 * @param[in]  Data table
 * @return     Error code
 */
bsp_status_t bsp_touch_get_data(uint8_t *touch_data, size_t size)
{
    bsp_status_t error;

    error = bsp_i2c_read (touch_handle, BSP_I2C_SLAVE_ADDR_TOUCH, FT5206_DEVICE_MODE, 1, touch_data, size);

    return error;
}

/**
 * @name       bsp_get_finger_position
 * @brief      Retrieve and format the x,y position of requested finger
 * @param[in]  Touch data pointer
 * @param[in]  Finger ID (position)
 * @param[in]  x data pointer
 * @param[in]  y data pointer
 * @return     Error code
 */
bsp_status_t bsp_touch_get_position(uint8_t *touch_data, uint8_t finger_id, uint16_t *x, uint16_t *y)
{
    if(finger_id >= 1 && finger_id <= 5)
    {
        // Form X1 touch position data to 16bits
        *x = ( (touch_data[FT5206_TOUCH_XH(finger_id)] & FT5206_TOUCH_XH_MSB_X_POS) << 8)
            + touch_data[FT5206_TOUCH_XL(finger_id)];

        // Form Y1 touch position data to 16bits
        *y = ( (touch_data[FT5206_TOUCH_YH(finger_id)] & FT5206_TOUCH_YH_MSB_Y_POS) << 8)
            + touch_data[FT5206_TOUCH_YL(finger_id)];
    }
    else
    {
        *x = 0;
        *y = 0;
    }

    if(*x > 799)
        *x = 799;

    if(*y > 479)
        *y = 479;

    return BSP_STATUS_OK;
}

/**
 * @name       bsp_touch_get_finger_event
 * @brief      Check if there is an finger event detected
 * @param[in]  Touch data pointer
 * @param[in]  Finger ID (position)
 * @param[in]  Finger event data pointer 
 * @return     Error code
 */
bsp_status_t bsp_touch_get_finger_event(uint8_t *touch_data, uint8_t finger_id, uint8_t *event)
{
    if(finger_id >= 1 && finger_id <= 5)
    {
    	*event = ((touch_data[FT5206_TOUCH_XH(finger_id)] & 0xC0) >> 6);
    }
    return BSP_STATUS_OK;
}

/**
 * @name       bsp_touch_get_finger_id
 * @brief      Check if there is an finger id detected
 * @param[in]  Touch data pointer
 * @param[in]  Finger ID (position)
 * @param[in]  Finger id pointter
 * @return     Error code
 */
bsp_status_t bsp_touch_get_finger_id(uint8_t *touch_data, uint8_t finger_id, uint8_t *id)
{
    if(finger_id >= 1 && finger_id <= 5)
    {
    	*id = ((touch_data[FT5206_TOUCH_YH(finger_id)] & 0xF0) >> 4);
    }
    return BSP_STATUS_OK;
}

/**
 * @name       bsp_touch_write_reg
 * @brief      Write touchscreen register
 * @param[in]  Register address
 * @param[in]  value to be written
 * @return     Error code
 */
bsp_status_t bsp_touch_write_reg(uint8_t address, uint8_t data)
{
    bsp_status_t error;

    error = bsp_i2c_write (touch_handle, BSP_I2C_SLAVE_ADDR_TOUCH, address, 1, &data, 1 );

    // Return status code
    return error;
}

/**
 * @name       bsp_touch_read_reg
 * @brief      Read touchscreen register
 * @param[in]  Register address
 * @param[in]  Value to receive data read
 * @return     Error code
 */
bsp_status_t bsp_touch_read_reg(uint8_t address, uint8_t *data)
{
    bsp_status_t error;

    error = bsp_i2c_read (touch_handle, BSP_I2C_SLAVE_ADDR_TOUCH, address, 1, data, 1 );

    // Return status code
    return error;
}

/**
 * @name       bsp_touch_read_reg
 * @brief      Read touchscreen register
 * @param[in]  Register address
 * @param[in]  Value to receive data read
 * @param[in]  data size to be read
 * @return     Error code
 */
bsp_status_t bsp_touch_read_regs(uint8_t address, uint8_t *data, size_t data_size)
{
    bsp_status_t error;

    error = bsp_i2c_read (touch_handle, BSP_I2C_SLAVE_ADDR_TOUCH, address, 1, data, data_size);

    // Return status code
    return error;
}

/*******************************************************************************************************************//**
 * Touch IRQ callback function
 *
 * NOTE: This function will return to the highest priority available task.
 *
 * @param      p_args          IRQ callback data
 **********************************************************************************************************************/
void touch_irq_cb(external_irq_callback_args_t * p_args)
{
    if(6 == p_args->channel)
     {
        BaseType_t context_switch = pdFALSE;

        /* Set touch IRQ semaphore */
        xSemaphoreGiveFromISR(g_touch_semaphore, &context_switch);

        /* Return to the highest priority available task */
        portYIELD_FROM_ISR(context_switch);
    }
}
