/**
 * @file m13_ra6m3_ek_vcom.c
 * @brief M13-RA6M3-EK Virtual COM definition
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#include "r_sci_uart.h"
#include "m13_ra6m3_ek_vcom.h"
#include "m13_ra6m3_ek_status.h"
#include <stdarg.h>

//VCOM IOPORT structure
static ioport_instance_ctrl_t vcom_pin_ctrl;

// Macro definitions

int vfprintf_custom(void *out, const char *format, va_list args);

// UART control
sci_uart_instance_ctrl_t vcom_ctrl;

baud_setting_t vcom_baud_setting =
{
     .semr_baudrate_bits_b.abcse = 0,
     .semr_baudrate_bits_b.abcs  = 0,
     .semr_baudrate_bits_b.bgdm  = 1,
     .cks   = 0,
     .brr   = 64,
     .mddr  = (uint8_t) 256,
     .semr_baudrate_bits_b.brme  = false
};

// UART extended configuration for UARTonSCI HAL driver
const sci_uart_extended_cfg_t vcom_cfg_extend =
{
     .clock             = SCI_UART_CLOCK_INT,
     .rx_edge_start     = SCI_UART_START_BIT_FALLING_EDGE,
     .noise_cancel      = SCI_UART_NOISE_CANCELLATION_DISABLE,
     .rx_fifo_trigger   = SCI_UART_RX_FIFO_TRIGGER_MAX,
     .p_baud_setting    = &vcom_baud_setting,

#if ( FSP_VERSION_MAJOR <= 2 )
     .uart_mode         = UART_MODE_RS232,
     .ctsrts_en         = SCI_UART_CTSRTS_RTS_OUTPUT,
     .flow_control_pin  = (bsp_io_port_pin_t) (0xFFFFU),
#else
     .flow_control    = SCI_UART_FLOW_CONTROL_RTS,
     .flow_control_pin  = (bsp_io_port_pin_t) UINT16_MAX,
#endif
};

// UART interface configuration
uart_cfg_t vcom_cfg =
{
     .channel       = 0,
     .data_bits     = UART_DATA_BITS_8,
     .parity        = UART_PARITY_OFF,
     .stop_bits     = UART_STOP_BITS_1,
     .p_callback    = NULL,
     .p_context     = NULL,
     .p_extend      = &vcom_cfg_extend,
     .p_transfer_tx = NULL,
     .p_transfer_rx = NULL,
     .rxi_ipl       = (12),
     .txi_ipl       = (12),
     .tei_ipl       = (12),
     .eri_ipl       = (12),
     .rxi_irq       = FSP_INVALID_VECTOR,
     .txi_irq       = FSP_INVALID_VECTOR,
     .tei_irq       = FSP_INVALID_VECTOR,
     .eri_irq       = FSP_INVALID_VECTOR,
};

// UART interface module
const uart_instance_t g_vcom =
{
     .p_ctrl = &vcom_ctrl,
     .p_cfg = &vcom_cfg,
     .p_api = &g_uart_on_sci
};

/**
 * @brief      Configure M13-RZA2M-EK virtual COM pins
 * @param[in]  None
 * @return     None
 */
void bsp_vcom_init(void)
{
    ioport_cfg_t vcom_pin_cfg;

    vcom_pin_cfg.number_of_pins = sizeof(m13_ra6m3_ek_vcom_pin_data)/sizeof(ioport_pin_cfg_t);
    vcom_pin_cfg.p_pin_cfg_data = &m13_ra6m3_ek_vcom_pin_data[0];

    //Configure vcom pins
    R_IOPORT_Open(&vcom_pin_ctrl, &vcom_pin_cfg);
}

/**
 * @brief      Open and configure the uart channel
 * @param[in]  channel : The desired channel to be configurated
 * @return     bsp_status_t : Error code (see #ErrorCode enumeration in )
 */
bsp_status_t bsp_vcom_open(bsp_vcom_channel_t channel)
{
    fsp_err_t error;

    vcom_cfg.channel = channel;

    error = R_SCI_UART_Open(&vcom_ctrl, &vcom_cfg);


    if(error == FSP_SUCCESS)
    {
        return BSP_STATUS_OPEN_FAILURE;
    }
    else
    {
        return BSP_STATUS_OK;
    }
}


